package org.ow2.isac.plugin.ldapinjector;

import org.ow2.clif.scenario.isac.plugin.SessionObjectAction;
import java.security.Security;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import org.ow2.clif.scenario.isac.plugin.DataProvider;
import org.ow2.clif.scenario.isac.exception.IsacRuntimeException;
import java.io.UnsupportedEncodingException;
import java.lang.Error;
import java.util.Map;
import org.ow2.clif.storage.api.ActionEvent;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.scenario.isac.plugin.SampleAction;
import org.ow2.clif.scenario.isac.util.ParameterParser;
import com.novell.ldap.LDAPAttribute;
import com.novell.ldap.LDAPAttributeSet;
import com.novell.ldap.LDAPConnection;
import com.novell.ldap.LDAPEntry;
import com.novell.ldap.LDAPException;
import com.novell.ldap.LDAPJSSESecureSocketFactory;
import com.novell.ldap.LDAPModification;
import com.novell.ldap.LDAPSearchResults;
import com.novell.ldap.LDAPSocketFactory;
import com.novell.ldap.util.Base64;

/**
 * Implementation of a session object for plugin ~LdapInjector~
 * 
 * * @author Thomas Escalle
 */
public class SessionObject implements SessionObjectAction, DataProvider, SampleAction {

	static final String PLUGIN_LDAPHOST = "ldapHost";
	static final String PLUGIN_LDAPVERSION = "ldapVersion";
		
	static final int SAMPLE_CONNECTION = 0;
	static final String SAMPLE_CONNECTION_PASSWORD = "password";
	static final String SAMPLE_CONNECTION_LOGIN = "login";
	static final String SAMPLE_CONNECTION_LDAPPORT = "ldapPort";

	static final int SAMPLE_SSLCONNECTION = 1;
	static final String SAMPLE_SSLCONNECTION_PASSWORD = "password";
	static final String SAMPLE_SSLCONNECTION_LOGIN = "login";
	static final String SAMPLE_SSLCONNECTION_LDAPPORT = "ldapPort";
	static final String SAMPLE_SSLCONNECTION_KEYSTOREPATH = "keystorePath";
	
	static final int SAMPLE_SEARCH = 2;
	static final String SAMPLE_SEARCH_SEARCHSCOPE = "searchScope";
	static final String SAMPLE_SEARCH_SEARCHFILTER = "searchFilter";
	static final String SAMPLE_SEARCH_SEARCHBASE = "searchBase";
	
	static final int SAMPLE_CLOSECONNECTION = 3;

	static final int SAMPLE_ADDENTRY = 4;
	static final String SAMPLE_ADDENTRY_ENTRYNODETYPE = "entryNodeType";
	static final String SAMPLE_ADDENTRY_ENTRYNODENAME = "entryNodeName";
	static final String SAMPLE_ADDENTRY_ATTRIBUTESLIST = "AttributesList";
	static final String SAMPLE_ADDENTRY_LDAPDN = "ldapDN";
	
	static final int SAMPLE_DELETEENTRY = 5;
	static final String SAMPLE_DELETEENTRY_LDAPDN = "ldapDN";
	
	static final int SAMPLE_ADDATTRIBUTE = 6;
	static final String SAMPLE_ADDATTRIBUTE_ATTRIBUTESNAMEVALUES = "attributesNameValues";
	static final String SAMPLE_ADDATTRIBUTE_DN = "dn";
	
	static final int SAMPLE_DELETEATTRIBUTE = 7;
	static final String SAMPLE_DELETEATTRIBUTE_ATTRIBUTESTODELETE = "attributesToDelete";
	static final String SAMPLE_DELETEATTRIBUTE_DN = "dn";
	
	static final String CONNECT_TYPE = "CONNECT";
	static final String CONNECT_SSL_TYPE = "SSL_CONNECT";
	static final String DISCONNECT_TYPE = "DISCONNECT";
	static final String SEARCH_TYPE = "SEARCH";
	static final String ADDENTRY_TYPE = "ADD_ENTRY";
	static final String DELETEENTRY_TYPE = "DELETE_ENTRY";
	static final String ADDATTRIBUTE_TYPE = "ADD_ATTRIBUTE";
	static final String DELETEATTRIBUTE_TYPE = "DELETE_ENTRY";


	private String hostname;
	private String loginDn;
	private String password;
	private int ldapVersion;
	
	private LDAPConnection ldapConnection;

	/**
	 * Constructor for specimen object.
	 *
	 * @param params key-value pairs for plugin parameters
	 * @throws ClifException 
	 * @throws ClifException 
	 * @throws ClifException 
	 */
	public SessionObject(Hashtable params) throws ClifException {
		// local address setting
		String value = (String)params.get(PLUGIN_LDAPHOST);
		// System.out.println("Le ldapHost est : "+value);
		if (value != null && value.length() > 0)
		{
			try
			{
				hostname = value;
			}
			catch (Exception ex)
			{
				throw new ClifException("ISAC can't get HostName because the specified hostname is not valid: " + value, ex);
			}
		}
		else
		{
			throw new ClifException("ISAC can't get HostName because the specified hostname is not valid: " + value);
		}		

		// loginDN setting
//		value = (String)params.get(OLD1_PLUGIN_LOGINDN);
//		// System.out.println("Le loginDN est : "+value);
//		if (value != null && value.length() > 0)
//		{
//			try
//			{
//				loginDn = value;
//			}
//			catch (Exception ex)
//			{
//				throw new ClifException("ISAC can't get loginDN because the specified loginDN is not valid: " + value, ex);
//			}
//		}
//		else
//		{
//			throw new ClifException("ISAC can't get loginDN because the specified loginDN is not valid: " + value);
//		}	
//
//		// password setting
//		value = (String)params.get(OLD0_PLUGIN_PASSWORD);
//		// System.out.println("Le password est : "+value);
//		if (value != null && value.length() > 0)
//		{
//			try
//			{
//				password = value;
//			}
//			catch (Exception ex)
//			{
//				throw new ClifException("ISAC can't get password because the specified password is not valid: " + value, ex);
//			}
//		}
//		else
//		{
//			throw new ClifException("ISAC can't get password because the specified password is not valid: " + value);
//		}

		// ldap version setting
		value = (String)params.get(PLUGIN_LDAPVERSION);
		// System.out.println("La version ldap est : "+value);
		if (value != null && value.length() > 0)
		{
			try
			{
				ldapVersion = Integer.parseInt(value);
			}
			catch (Exception ex)
			{
				throw new ClifException("ISAC can't get ldap version because the specified ldap version is not valid: " + value, ex);
			}
		}
		else
		{
			throw new ClifException("ISAC can't get ldap version because the specified ldap version is not valid: " + value);
		}
	}

	/**
	 * Copy constructor (clone specimen object to get session object).
	 *
	 * @param so specimen object to clone
	 */
	private SessionObject(SessionObject so) {
		this.hostname = so.hostname;
		this.ldapVersion = so.ldapVersion;
		//this.loginDn = so.loginDn;
		//this.password = so.password;
	}


	////////////////////////////////////////
	// SessionObjectAction implementation //
	////////////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#createNewSessionObject()
	 */
	public Object createNewSessionObject() {
		return new SessionObject(this);
	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#close()
	 */
	public void close() {

	}

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SessionObjectAction#reset()
	 */
	public void reset() {

	}

	
	/////////////////////////////////
	// DataProvider implementation //
	/////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.DataProvider#doGet()
	 */
	public String doGet(String variable) {
		return null;
	}

	
	/////////////////////////////////
	// SampleAction implementation //
	/////////////////////////////////

	/**
	 * @see org.ow2.clif.scenario.isac.plugin.SampleAction#doSample()
	 */
	public ActionEvent doSample(int number, Map params, ActionEvent report) {
		switch (number) {
		case SAMPLE_DELETEATTRIBUTE:
				return doDeleteAttributes(number,params,report);
			case SAMPLE_ADDATTRIBUTE:
				return doAddAttributes(number,params,report);
			case SAMPLE_DELETEENTRY:
				return doDeleteEntry(number,params,report);
			case SAMPLE_ADDENTRY:
				return doAddEntry(number,params,report);
			case SAMPLE_CONNECTION:
				return doConnection(number,params,report);
			case SAMPLE_SSLCONNECTION:
				return doSSLConnection(number,params,report);
			case SAMPLE_SEARCH:
				return doSearch(number,params,report);
			case SAMPLE_CLOSECONNECTION:
				return doCloseConnection(number,params,report);				
			default:
			throw new Error("Unable to find this sample in ~LdapInjector~ ISAC plugin: " + number);
		}
	}

	/**
	 * Does a connection to a LDAP directory (bind).
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */
	public ActionEvent doConnection(int number, Map params, ActionEvent report) {
        ldapConnection = new LDAPConnection();
        loginDn = (String) params.get(SAMPLE_CONNECTION_LOGIN);
        password = (String) params.get(SAMPLE_CONNECTION_PASSWORD);
        
		report.type = CONNECT_TYPE;
		report.setDate(System.currentTimeMillis());
        
        try {
           // connect to the server
        	ldapConnection.connect( hostname, Integer.parseInt((String) params.get(SAMPLE_CONNECTION_LDAPPORT)));
           // bind to the server
        	ldapConnection.bind(ldapVersion, loginDn, password.getBytes("UTF8"));
        	
        	report.duration = (int)(System.currentTimeMillis() - report.getDate());
            report.successful = true;
            report.comment = "Successful bind with server.";
            // System.out.println("Successful bind with server.");

        } catch( LDAPException e ) {
        	report.successful = false;
        	report.result = e.toString();
        	report.comment = "ISAC LdapInjector error occured";
        	// System.out.println("ISAC LdapInjector error occured "+password+". "+e.toString());
        } catch( UnsupportedEncodingException e ) {
        	report.successful = false;
        	report.result = e.toString();
        	report.comment = "ISAC LdapInjector can't UTF8 encode password "+password;
        	// System.out.println("ISAC LdapInjector can't UTF8 encode password "+password+". "+e.toString());
        }
		return report;
	}

	/**
	 * Does a SSL connection to a LDAP directory (bind).
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */
	public ActionEvent doSSLConnection(int number, Map params, ActionEvent report) {
		LDAPSocketFactory ssf;
        loginDn = (String) params.get(SAMPLE_SSLCONNECTION_LOGIN);
        password = (String) params.get(SAMPLE_SSLCONNECTION_PASSWORD);	
        
        try {
            // Dynamically set JSSE as a security provider
             Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());

            // Dynamically set the property that JSSE uses to identify
            // the keystore that holds trusted root certificates
             System.setProperty("javax.net.ssl.trustStore", (String) params.get(SAMPLE_SSLCONNECTION_KEYSTOREPATH));

             ssf = new LDAPJSSESecureSocketFactory();

            // Set the socket factory as the default for all future connections
             LDAPConnection.setSocketFactory(ssf);

            // Note: the socket factory can also be passed in as a parameter
            // to the constructor to set it for this connection only.
             ldapConnection = new LDAPConnection();

     		 report.type = CONNECT_SSL_TYPE;
    		 report.setDate(System.currentTimeMillis());             
             
            // connect to the server
             ldapConnection.connect(hostname, Integer.parseInt((String) params.get(SAMPLE_SSLCONNECTION_LDAPPORT)) );

            // authenticate to the server
             ldapConnection.bind(ldapVersion, loginDn, password.getBytes("UTF8"));
             
         	 report.duration = (int)(System.currentTimeMillis() - report.getDate());
             report.comment = "Successful SSL bind with server.";
             report.successful = true;
             // System.out.println("Successful SSL bind with server.");

         } catch ( LDAPException e ) {
         	report.successful = false;
        	report.result = e.toString();
        	report.comment = "ISAC LdapInjector error occured.";
        	// System.out.println("ISAC LdapInjector error occured. "+e.toString()); 
         } catch ( UnsupportedEncodingException e ) {
        	report.successful = false;
        	report.result = e.toString();
        	report.comment = "ISAC LdapInjector can't UTF8 encode password "+password;
        	// System.out.println("ISAC LdapInjector can't UTF8 encode password "+password+". "+e.toString()); 
         }	

         return report;
	}

	
	/**
	 * Close an opened connection to a LDAP directory (bind).
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */
	public ActionEvent doCloseConnection(int number, Map params, ActionEvent report)
	{
		if (ldapConnection != null)
		{
			
			report.type = DISCONNECT_TYPE;
			report.setDate(System.currentTimeMillis());  
			
			try {
				// disconnect with the server
	            ldapConnection.disconnect();
	            
	            report.duration = (int)(System.currentTimeMillis() - report.getDate());
	            
				ldapConnection = null;
				report.successful = true;
				report.comment = "ISAC LdapConnection disconnection Successful";
				// System.out.println("ISAC LdapConnection disconnection Successful");
				return report;				
			} catch (LDAPException e) {
				report.successful = false;
				report.comment = "ISAC LdapInjector can't disconnect from " + hostname;
				report.result = e.toString();
				// System.out.println("ISAC LdapInjector can't disconnect from " + hostname + ". " +e.toString());
				return report;
			}

		} else {
			report.successful = false;
			report.comment = "ISAC LdapInjector can't disconnect unopen ldap connection";
			report.result = "ignored";
			// System.out.println("ISAC LdapInjector can't disconnect unopen ldap connection.");
			return null;
		}
	}	

	/**
	 * Does search into a LDAP directory.
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */
	public ActionEvent doSearch(int number, Map params, ActionEvent report) {
        try {
        	
			report.type = SEARCH_TYPE;
			report.setDate(System.currentTimeMillis());
        	
            LDAPSearchResults searchResults = 
            	ldapConnection.search(  (String) params.get(SAMPLE_SEARCH_SEARCHBASE),

            			    Integer.parseInt((String) params.get(SAMPLE_SEARCH_SEARCHSCOPE)),

            				(String) params.get(SAMPLE_SEARCH_SEARCHFILTER),

                             null,         // return all attributes


                             false);       // return attrs and values
		
            //printSearchResult(searchResults);
            
            report.duration = (int)(System.currentTimeMillis() - report.getDate()); 
    	    report.successful = true;
    	    report.comment = "LDAP Search succeded";
    	    // System.out.println("LDAP Search succeded");
	        
	    } catch( LDAPException e ) {
         	report.successful = false;
        	report.result = e.toString();
        	report.comment = "ISAC LdapInjector error occured";
        	// System.out.println("ISAC LdapInjector error occured. "+e.toString());
	    }

		return report;
	}
	
	/**
	 * Add an entry into a LDAP directory. The connection to the LDAP directory must be established before doing the doAddEntry.
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */	
	public ActionEvent doAddEntry(int number, Map params, ActionEvent report) {
		String dn = (String) params.get(SAMPLE_ADDENTRY_LDAPDN);
		String insertNodeNameValue =
			ParameterParser.getCombo((String) params.get(SAMPLE_ADDENTRY_ENTRYNODETYPE))
			+ "="
			+ (String) params.get(SAMPLE_ADDENTRY_ENTRYNODENAME);
		List<String> attributesList = ParameterParser.getNField(
			(String) params.get(SAMPLE_ADDENTRY_ATTRIBUTESLIST));
		
		LDAPAttributeSet attributeSet = new LDAPAttributeSet();
		
		// Test to know if the entry node belongs to the attributesList. If no, then we put it in this List.
		if (!attributesList.contains(insertNodeNameValue))
			attributesList.add(insertNodeNameValue);

		// Test to know if the entry node belongs to the dn. If no, then we add it at the beginning of the dn.
		if (dn.indexOf(insertNodeNameValue) == -1)
			dn = insertNodeNameValue+","+dn;
		
		String name = null;
		String[] values = null;
		
		String[] tabParam_NameValue = attributesList.toArray(new String[attributesList.size()]);
		for (int i = 0; i<tabParam_NameValue.length; i++) {
			name = tabParam_NameValue[i].substring(0, tabParam_NameValue[i].indexOf("="));
			values = (tabParam_NameValue[i].substring(tabParam_NameValue[i].indexOf("=")+1, tabParam_NameValue[i].length())).split(",");

			attributeSet.add( new LDAPAttribute( name, values ));
		}
		LDAPEntry newEntry = new LDAPEntry( dn, attributeSet );

		report.type = ADDENTRY_TYPE;
		report.setDate(System.currentTimeMillis());
		
        try {
			ldapConnection.add( newEntry );
			
			report.duration = (int)(System.currentTimeMillis() - report.getDate()); 
	        report.successful = true;
	        report.comment = "Added object: " + dn + " successfully.";
			// System.out.println( "Added object: " + dn + " successfully." );				
			
		} catch (LDAPException e) {
			report.successful = false;
			report.result = e.toString();
			report.comment = "ISAC Ldap Injector can't add object "+dn+" to the ldap directory.";
            // System.out.println("ISAC Ldap Injector can't add object "+dn+" to the ldap directory. "+e.toString());
		} catch (Exception e) {
			report.successful = false;
			report.result = e.toString();
			report.comment = "ISAC Ldap Injector can't add object "+dn+" to the ldap directory.";
            // System.out.println("ISAC Ldap Injector can't add object "+dn+" to the ldap directory. "+e.toString());			
		}

		return report;
	}

	/**
	 * Delete an entry from a LDAP directory. The connection to the LDAP directory must be established before doing the doDeleteEntry.
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */	
	public ActionEvent doDeleteEntry(int number,Map params, ActionEvent report) {
		String dn = (String) params.get(SAMPLE_DELETEENTRY_LDAPDN);
		
		report.type = DELETEENTRY_TYPE;
		report.setDate(System.currentTimeMillis());
		
		try {
			
			ldapConnection.delete(dn);

			report.duration = (int)(System.currentTimeMillis() - report.getDate()); 
			report.successful = true;
			report.comment = "Entry: " + dn + " was deleted." ;
			// System.out.println("Entry: " + dn + " was deleted.");
			
		} catch (LDAPException e) {
			report.successful = false;
			report.comment = "ISAC Ldap Injector can't delete dn : "+dn;
            if ( e.getResultCode() == LDAPException.NO_SUCH_OBJECT ) {
            	report.result = "Error: No such object";
                // System.out.println( "Error: No such object" );
			} else if ( e.getResultCode() == LDAPException.INSUFFICIENT_ACCESS_RIGHTS ) {
				report.result = "Error: Insufficient rights";
                // System.out.println( "Error: Insufficient rights" );
			} else {
				report.comment = "Error: " + e.toString();
                // System.out.println( "Error: " + e.toString() );
			}    
		}
		
		return report;
	}

	/**
	 * Add attributes into an entry of LDAP directory. The connection to the LDAP directory must be established before doing the doAddAttributes.
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */	
	public ActionEvent doAddAttributes(int number, Map params, ActionEvent report) {
        String dn = (String) params.get(SAMPLE_ADDATTRIBUTE_DN);
        String[] attributesNameValue = (String[])
        	ParameterParser.getNField((String) params.get(SAMPLE_ADDATTRIBUTE_ATTRIBUTESNAMEVALUES)).toArray();

        ArrayList<LDAPModification> modList = new ArrayList<LDAPModification>();
		LDAPAttribute attribute = null;
  
		String name = null;
		String[] values = null;      		
		
		for (int i = 0; i<attributesNameValue.length; i++) {
 			name = attributesNameValue[i].substring(0, attributesNameValue[i].indexOf("="));
 			values = (attributesNameValue[i].substring(attributesNameValue[i].indexOf("=")+1, attributesNameValue[i].length())).split(",");		
 			attribute = new LDAPAttribute(name, values);
	        modList.add( new LDAPModification(LDAPModification.ADD,attribute));
		}   		

        LDAPModification[] mods = new LDAPModification[modList.size()]; 
        mods = (LDAPModification[])modList.toArray(mods);	

		report.type = ADDATTRIBUTE_TYPE;
		report.setDate(System.currentTimeMillis());        
        
        try {
            ldapConnection.modify(dn, mods);

			report.duration = (int)(System.currentTimeMillis() - report.getDate());             
            report.successful = true;
            report.comment = "Successfully add the attribute.";
            // System.out.println("Successfully add the attribute.");        
        
        } catch( LDAPException ex ) {
        	
        	report.successful = false;
        	
            if(ex.getResultCode()== LDAPException.ATTRIBUTE_OR_VALUE_EXISTS){ 
            	report.result = ex.toString();
            	report.comment = "Failed to add existing attribute.";
                // System.out.println("Failed to add existing attribute : "+ex.toString());
            } else {
            	report.result = ex.toString();
            	report.comment = "Failed to add attribute.";            	
            	// System.out.println("Failed to add attribute : "+ex.toString());
            }
        } catch ( Exception e) {
        	report.result = e.toString();
        	report.comment = "Error : Failed to add attribute.";            	
        	// System.out.println("Error : Failed to add attribute : "+e.toString());        	
        }
        
        return report;
	}

	/**
	 * Delete an attributes of an entry of LDAP directory. The connection to the LDAP directory must be established before doing the doDeleteAttributes.
	 *
	 * @param number : The number which handles the connection to the ldap directory.
	 * @param report : The ActionReport to update.
	 * @param params : A Map containing all the useful variable.
	 * @return the report.
	 */	
	public ActionEvent doDeleteAttributes(int number, Map params, ActionEvent report) {

        String dn = (String) params.get(SAMPLE_DELETEATTRIBUTE_DN);
        String[] attributesToDelete = (String[])ParameterParser.getNField(
        	(String) params.get(SAMPLE_DELETEATTRIBUTE_ATTRIBUTESTODELETE)).toArray();

        ArrayList<LDAPModification> modList = new ArrayList<LDAPModification>();
 
		LDAPAttribute attribute = null;		

		for (int i = 0; i<attributesToDelete.length; i++) {
			attribute = new LDAPAttribute(attributesToDelete[i]);
	        modList.add( new LDAPModification(LDAPModification.DELETE,attribute));
		}    
		
        // Create modifications array
        LDAPModification[] modsdel = new LDAPModification[modList.size()]; 
        modsdel = (LDAPModification[])modList.toArray(modsdel);	

		report.type = DELETEATTRIBUTE_TYPE;
		report.setDate(System.currentTimeMillis());          
        
        try {
			ldapConnection.modify(dn, modsdel);

			report.duration = (int)(System.currentTimeMillis() - report.getDate()); 			
	        report.successful = true;
	        report.comment = "Successfully delete the attribute.";	
	        // System.out.println("Successfully delete the attribute.");			
		} catch( LDAPException ex ) {
        	report.successful = false;
        	report.result = ex.toString();
        	report.comment = "Failed to delete attribute";            	
        	// System.out.println("Failed to delete attribute : "+ex.toString());
		}    

		return report;
	}
	
	///////////////////////////////
	// TestAction implementation //
	///////////////////////////////

	/*#** 
	 * @see org.ow2.clif.scenario.isac.plugin.TestAction#doTest()
	 *#*/
	public boolean doTest(int number,Map params){
	  throw new IsacRuntimeException("No action defined for this test in ~LdapInjector~ ISAC plugin: " + number);
	}
	
	/**
	 * Print out the search results in LDAP directory.
	 *
	 * @param searchResults : The result of the method LDAPConnection.search()
	 *
	 */		
	private void printSearchResult(LDAPSearchResults searchResults) {
		// This while loop goes through all the entries
        while ( searchResults.hasMore()) {
            LDAPEntry nextEntry = null;
            try {
                nextEntry = searchResults.next();
            } catch(LDAPException e) {
                // System.out.println("Error : " + e.toString());
               // Exception is thrown, go for next entry
                if(e.getResultCode() == LDAPException.LDAP_TIMEOUT || e.getResultCode() == LDAPException.CONNECT_ERROR)
                   break;
                else
                   continue;
            }

            // System.out.println("\n" + nextEntry.getDN());
            // System.out.println("  Attributes: ");

            LDAPAttributeSet attributeSet = nextEntry.getAttributeSet();
            Iterator allAttributes = attributeSet.iterator();
            // This while loop goes through all the attributes
            while(allAttributes.hasNext()) {
                LDAPAttribute attribute = (LDAPAttribute)allAttributes.next();
                String attributeName = attribute.getName();

                // System.out.println("    " + attributeName);

                Enumeration allValues = attribute.getStringValues();

                if( allValues != null) {
                	// This while loop goes through all the attribute values
                    while(allValues.hasMoreElements()) {
                        String Value = (String) allValues.nextElement();
                        if (Base64.isLDIFSafe(Value)) {
                           // is printable
                            // System.out.println("      " + Value);
                        } else {
                           // base64 encode and then print out
                            Value = Base64.encode(Value.getBytes());
                            // System.out.println("      " + Value);
                        }
                    }
                }
            }
        }		
	}
}
