/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.api.graph;

import java.util.List;
import java.util.Map;

import org.jfree.chart.JFreeChart;
import org.ow2.clif.analyze.lib.graph.ChartData;
import org.ow2.clif.analyze.lib.graph.DatasetOptions;
import org.ow2.clif.analyze.lib.graph.StatisticDataImpl;

/**
 * The Interface ExportControl.
 * @author Jordan BRUNIER
 * @author Gregory CALONNIER
 */
public interface ExportControl {

//  Fractal interface name
    /** The Constant EXPORT_CONTROL. */
    static public final String EXPORT_CONTROL = "Export control";

    /**
     * Export to HTML.
     * 
     * @param fileName the file name
     * @param chartId the chart id
     * @param imageFormat the image format
     * @param comment the comment
     * @param statistics the statistics
     * @param title the title
     * 
     * @return true, if successful
     */
    public boolean exportToHTML(String fileName, int chartId, String imageFormat, String comment, Map<String, Map<String, Double>> statistics, String title);
    
    
    /**
     * Export to HTML.
     * 
     * 
     */
	/**
	 * @param fileName
	 * @param statisticMap
	 * @param commentMap
	 * @param jFreeChartMap
	 * @param imageType
	 * @param pageIdAndDatasetIds
	 * @param chartDataList
	 * @param datasetIdAndGraphOptions
     * @return true, if successful
	 */
	public boolean exportToHTML(
			String fileName,
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap,
			String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions);

    /**
     * Export to XML.
     * 
     * @param fileName the file name
     * @param chartId the chart id
     * @param imageFormat the image format
     * @param comment the comment
     * @param statistics the statistics
     * @param title the title
     * 
     * @return true, if successful
     */
    public boolean exportToXML(String fileName, int chartId, String imageFormat, String comment, Map<String, Map<String, Double>> statistics, String title);
    
    /**
     * Export to XML
     */
    public boolean exportToXML(
    		String fileName, 
    		Map<Integer, List<StatisticDataImpl>> statisticMap,
    		Map<String, String> commentMap,
    		Map<Integer, JFreeChart> jFreeChartMap,
    		String imageType,
    		Map<Integer, List<Integer>> pageIdAndDatasetIds,
    		Map<Integer, ChartData> chartDataList,
    		Map<Integer, DatasetOptions> datasetIdAndGraphOptions);
    

    /**
     * Export to text
     * 
     * @param fileName
     * @param statisticMap
     * @param commentMap
     * @return true if successful
     */
    public boolean exportToText(
    		String fileName, 
    		Map<Integer, List<StatisticDataImpl>> statisticMap,
    		Map<String, String> commentMap,
    		Map<Integer, JFreeChart> jFreeChartMap,
    		String imageType);
    
    
    /**
	 * Save Routine As XML
	 */
	public boolean saveRoutineAsXML(
			String fileName, 
			Map<Integer, List<StatisticDataImpl>> statisticMap,
			Map<String, String> commentMap,
			Map<Integer, JFreeChart> jFreeChartMap,
			String imageType,
			Map<Integer, List<Integer>> pageIdAndDatasetIds,
			Map<Integer, ChartData> chartDataList,
			Map<Integer, DatasetOptions> datasetIdAndGraphOptions);
    
}

