/*
* CLIF is a Load Injection Framework
* Copyright (C) 2008 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Olivier Liu
*
*
*/
package org.ow2.clif.analyze.lib.graph;


import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.virtualdata.AggregateDataControl;
import org.ow2.clif.storage.api.BladeEvent;
/**
 * The Class AggregateDataImpl
 */
public class AggregateDataImpl implements BindingController, AggregateDataControl {

    /** The instance. */
    private static AggregateDataImpl instance= new AggregateDataImpl();
    
    /** The interface list. */
    static private String[] itfList = new String[] { DataSupplier.DATA_SUPPLIER };

    // field for client binding (access to stored data)
    /** The data supplier interface. */
    private DataSupplier dsItf;
    
    /**
     * Instantiates a new chart engine impl.
     */
    public AggregateDataImpl()
    { 
       //empty
    }
    
    
    /**
     * Gets the single instance of FilterDataImpl.
     * 
     * @return single instance of FilterDataImpl
     */
    public AggregateDataImpl getInstance()
    {
        return instance;
    }


    /////////////////////////////////
    // interface BindingController //
    /////////////////////////////////
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = (DataSupplier)serverItf;
        }
    }
    

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() 
    {
        return itfList;
    }

    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            return dsItf;
        }
        else
        {
            return null;
        }
    }

    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = null;
        }
    }
    
    ////////////////////////////////////
    // interface AggregateDataControl //
    ////////////////////////////////////
    
    /* (non-Javadoc)
     * @see org.objectweb.clif.analyze.api.graph.AggregateDataControl#aggregateBlades(java.lang)
     */
    public List<BladeEvent>  aggregateBladesOneSameEvent(List<List<BladeEvent>> bladeList){
    	Iterator<List<BladeEvent>> bladeEventListIterator = bladeList.iterator();
    	Comparator<BladeEvent> SORT_BY_TIME = new DateComp();
		TreeSet<BladeEvent> bladeTreeSet = new TreeSet<BladeEvent>(SORT_BY_TIME);
    	//get the blade event list
    	while (bladeEventListIterator.hasNext()) {
			List<BladeEvent> bladeEventList = bladeEventListIterator.next();
			Iterator<BladeEvent> bladeEventIterator = bladeEventList.iterator();
			//get each value from a bladeEvent then add to the blade result
			//treeSet : no duplicate elements + sort elements by time 
			while (bladeEventIterator.hasNext()) {
				BladeEvent bladeEvent = (BladeEvent) bladeEventIterator.next();
				bladeTreeSet.add(bladeEvent);
			}
		}
    	Iterator<BladeEvent> iterator2 = bladeTreeSet.iterator();
    	List<BladeEvent> bladeEventListResult = new ArrayList<BladeEvent>();
    	//treeSet to arrayList
    	while (iterator2.hasNext()) {
			BladeEvent bladeEvent = (BladeEvent) iterator2.next();
			bladeEventListResult.add(bladeEvent);
		}
    	return bladeEventListResult;
    }
    
    /////////////////////////////////
    // Class DateComp //
    /////////////////////////////////
    private class DateComp implements Comparator<BladeEvent> {
    	//sort by time
    	public int compare(BladeEvent bladeEvent1, BladeEvent bladeEvent2){
    		if(bladeEvent1.getDate()<bladeEvent2.getDate()){
    			return -1;
    		}
    		else if(bladeEvent1.getDate()>bladeEvent2.getDate()){
    			return 1;
    		}
    		else{
    			return 0;
    		}
    	}
    }


}
