/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.datasource.RawDataRead;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.BladeFilter;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.storage.api.StorageRead;
import org.ow2.clif.storage.api.TestDescriptor;
import org.ow2.clif.storage.lib.util.DateEventFilter;
import org.ow2.clif.storage.lib.util.NameBladeFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * The Class AnalyzerStorageProxyImpl.
 */
public class AnalyzerStorageProxyImpl implements BindingController, RawDataRead {

	private int LIFECYCLE_RUNNING_ROW = 3;
	private int ONE_VALUE = 0;
	
    //  list of client interfaces names
    /** The interface list. */
    static private String[] itfList = new String[] { 
        StorageRead.STORAGE_READ};


    // field for client binding (access to stored data)
    /** The raw data read interface. */
    private StorageRead srItf;

    /**
     * Instantiates a new analyzer storage proxy impl.
     */
    public AnalyzerStorageProxyImpl()
    {
        
    }
    
    /////////////////////////////////
    // interface BindingController //
    /////////////////////////////////


    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(StorageRead.STORAGE_READ))
        {
            srItf = (StorageRead) serverItf;
        }

    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() 
    {
        return itfList;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName)
    {
        if (clientItfName.equals(StorageRead.STORAGE_READ))
        {
            return srItf;
        }
        else
        {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
            IllegalBindingException, IllegalLifeCycleException 
    {
            if (clientItfName.equals(StorageRead.STORAGE_READ))
            {
                srItf = null;
            }
    }

    
    ///////////////////////////
    // interface RawDataRead //
    ///////////////////////////


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#countValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public long countValues(String testName, String bladeId, String eventTypeLabel, String YFieldLabel) 
    {
        try{
            return srItf.countEvents(testName, bladeId, eventTypeLabel, null);
        }catch(ClifException ce){
            return (long)0;
        }
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getBladesIds(java.lang.String)
     */
    public String[] getBladesIds(String testName) 
    {
        String[] bladesIds = null;
        
        try{
            BladeDescriptor[] blades = srItf.getTestPlan(testName, null);
            bladesIds = new String[blades.length]; 
            for (int i = 0; i < blades.length; i++) {
                bladesIds[i]=blades[i].getId();
            }
        }catch(ClifException ce){
            
        }
        return bladesIds;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getEventFieldLabels(java.lang.String, java.lang.String, java.lang.String)
     */
    public String[] getEventFieldLabels(String testName, String bladeId, String eventTypeLabel)
    {
        String[] eventFieldLabels = null;
        
        try{
            BladeEvent event = srItf.getEvents(testName, bladeId, eventTypeLabel, null, 0, -1)[0];
            eventFieldLabels = event.getFieldLabels();
        }catch(ClifException ce){
            
        }
        
        return eventFieldLabels;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getEventsTypeLabels(java.lang.String, java.lang.String)
     */
    public String[] getEventsTypeLabels(String testName, String bladeId) 
    {
        String[] eventsTypeLabels= null;
        try{
            BladeDescriptor blade = srItf.getTestPlan(testName, new NameBladeFilter(bladeId))[0];
            eventsTypeLabels = blade.getEventTypeLabels();
           // eventsTypeLabels = blade.
        }catch(ClifException ce){
            
        }
        return eventsTypeLabels;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getFieldsValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, int)
     */
    public Object[][] getFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start,int end, int count) 
    {
        Object[][] values = null;
        ArrayList valTemp1 = new ArrayList();
        ArrayList valTemp2 = new ArrayList();
        
        EventFilter filter = new DateEventFilter(start, end);
        try{
            if (count<0){
                count = (int)srItf.countEvents(testName, bladeId, eventTypeLabel, filter);
            }
            values = new Object[2][];
            BladeEvent[] events=srItf.getEvents(testName, bladeId, eventTypeLabel, filter, 0, count);
            for (int i = 0; i < events.length; i++) {
                valTemp1.add(events[i].getFieldValue(XFieldLabel));
                valTemp2.add(events[i].getFieldValue(YFieldLabel));
            }
            values[0] = valTemp1.toArray();
            values[1] = valTemp2.toArray();
            
            int i = values[0].length;
        } catch(ClifException ce){
     
        }
        return values;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getTestsNames()
     */
    public String[] getTestsNames() 
    {
        String[] testsNames = null;
        
        try{
            TestDescriptor[] tests = srItf.getTests(null);
            testsNames = new String[tests.length]; 
            for (int i = 0; i < tests.length; i++) {
                testsNames[i]=tests[i].getName();
            }
        }catch(ClifException ce){
            
        }
        return testsNames;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getMaxTimeFor(java.lang.String, java.lang.String)
     */
    public long getMaxTimeFor(String testName, String bladeId) 
    {
        long time = 0;
        try{
            BladeEvent[] events = srItf.getEvents(testName, bladeId, "lifecycle", null, -1, 1);
            time = events[0].getDate();
        }
        catch(ClifException ce)
        {
            System.out.println(ce);
        }
        return time;
    }


	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getTestPlan(java.lang.String, org.ow2.clif.storage.api.BladeFilter)
	 */
	public BladeDescriptor[] getTestPlan(String testName, BladeFilter filter) throws ClifException {
		try{
			return srItf.getTestPlan(testName, filter);
		}
		catch(ClifException ce){
//			System.out.println("getTestPlan failed");
		}
		return null;	
	}


	/* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getAllFieldsValues(...)
     */
	public Object[][] getAllFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count) {
		    Object[][] values = null;
	        ArrayList<Object> valTemp1 = new ArrayList<Object>();
	        ArrayList<Object> valTemp2 = new ArrayList<Object>();
	        try{
	            if (count<0){
	                count = (int)srItf.countEvents(testName, bladeId, eventTypeLabel, null);
	            }
	            values = new Object[2][];
	            BladeEvent[] events=srItf.getEvents(testName, bladeId, eventTypeLabel, null, 0, count);
	            for (int i = 0; i < events.length; i++) {
	                valTemp1.add(events[i].getFieldValue(XFieldLabel));
	                valTemp2.add(events[i].getFieldValue(YFieldLabel));
	            }
	            values[0] = valTemp1.toArray();
	            values[1] = valTemp2.toArray();
	        }
	        catch(ClifException ce){
	            
	        }
	        return values;
	}
    
	
	/* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getLifecycleRunning(java.lang.String, java.lang.String)
     */
    public long getLifecycleRunning(String testName, String bladeId) 
    {
    	long time = 0;
        try{
            BladeEvent[] events = srItf.getEvents(testName, bladeId, "lifecycle", null, LIFECYCLE_RUNNING_ROW, 1);
            time = events[0].getDate();
        }
        catch(ClifException ce)
        {
            
        }
        return time;
    }
    
    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getLifecycleCompleted(java.lang.String, java.lang.String)
     */
    public long getLifecycleCompleted(String testName, String bladeId) 
    {
        long time = 0;
        try{
            BladeEvent[] events = srItf.getEvents(testName, bladeId, "lifecycle", null, -1, 1);
            time = events[0].getDate();
        }
        catch(ClifException ce)
        {
            
        }
        return time;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getBladeEvents(java.lang.String, java.lang.String, java.lang.String, EventFilter, long, int)
     */
    public BladeEvent[] getBladeEvents(String testName,	String bladeId, String eventTypeLabel,EventFilter filter, long fromIndex,int count)throws ClifException {
    	try {
    		//get all the blade events.with no filter(null) from index 0 and all the value since count is -1 
    		BladeEvent[] bladeEventList = srItf.getEvents(testName, bladeId, eventTypeLabel, null, 0, -1);
    		return bladeEventList;
    	} 

    	catch (ClifException ce) {
  		System.out.println(ce);
    	}
    	return null;
    }
    
    /* (non-Javadoc) UTIL OU PAS ...
     * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getBladeEventsValues(BladeEvent[], java.lang.String, java.lang.String)
     */
    public Object[][] getBladeEventsValues(BladeEvent[] bladeEventList, String XFieldLabel, String YFieldLabel){
    	Object[][] values = null;
    	ArrayList<Object> valTemp1 = new ArrayList<Object>();
        ArrayList<Object> valTemp2 = new ArrayList<Object>();
        values = new Object[2][];
        for (int i = 0; i < bladeEventList.length; i++) {
            valTemp1.add(bladeEventList[i].getFieldValue(XFieldLabel));
            valTemp2.add(bladeEventList[i].getFieldValue(YFieldLabel));
        }
        values[0] = valTemp1.toArray();
        values[1] = valTemp2.toArray();
        
    	return values;
    }

	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.datasource.RawDataRead#getBladeDescriptor(java.lang.String, org.ow2.clif.storage.api.BladeFilter)
	 */
	public BladeDescriptor[] getBladeDescriptor(String testName, BladeFilter filter) {
		try {
			return srItf.getTestPlan(testName, filter);
		} catch (Exception e) {
			// TODO: handle exception
		}
		return null;
	}
    	  
	
}
