/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph;

import java.util.Map;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.datasource.RawDataRead;
import org.ow2.clif.analyze.api.graph.datasource.StatisticCollector;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.BladeEvent;
import org.ow2.clif.storage.api.BladeFilter;
import org.ow2.clif.storage.api.EventFilter;
import org.ow2.clif.supervisor.api.ClifException;

/**
 * The Class DataProxyImpl.
 */
public class DataProxyImpl implements DataSupplier, BindingController {

    //  list of client interfaces names
    /** The interface list. */
    static private String[] itfList = new String[] { 
        RawDataRead.RAW_DATA_READ,
        StatisticCollector.STATISTIC_COLLECTOR};


    // field for client binding (access to stored data)
    /** The raw data read interface. */
    private RawDataRead rdrItf;
    
    /** The statistic collectr interface. */
    private StatisticCollector scItf;
    
    /**
     * Instantiates a new data proxy impl.
     */
    public DataProxyImpl(){
        
    }
    
    ////////////////////////////
    // interface DataSupplier //
    ////////////////////////////


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getLifecycleRunning(java.lang.String, java.lang.String)
     */
    public long getLifecycleRunning(String testName, String bladeId){
    	return rdrItf.getLifecycleRunning(testName, bladeId);
    }
    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getLifecycleCompleted(java.lang.String, java.lang.String)
     */
    public long getLifecycleCompleted(String testName, String bladeId){
    	return rdrItf.getLifecycleCompleted(testName, bladeId);
    }
    
    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getAllFieldsValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public Object[][] getAllFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count){
    	return rdrItf.getAllFieldsValues(testName, bladeId, eventTypeLabel, XFieldLabel, YFieldLabel, start, end, count);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#countValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public long countValues(String testName, String bladeId, String eventTypeLabel, String YFieldLabel) 
    {
        return rdrItf.countValues(testName, bladeId, eventTypeLabel, YFieldLabel);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getBladesIds(java.lang.String)
     */
    public String[] getBladesIds(String testName) 
    {
        return rdrItf.getBladesIds(testName);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getEventFieldLabels(java.lang.String, java.lang.String, java.lang.String)
     */
    public String[] getEventFieldLabels(String testName, String bladeId, String eventTypeLabel) 
    {
        return rdrItf.getEventFieldLabels(testName, bladeId, eventTypeLabel);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getEventsTypeLabels(java.lang.String, java.lang.String)
     */
    public String[] getEventsTypeLabels(String testName, String bladeId) 
    {
        return rdrItf.getEventsTypeLabels(testName, bladeId);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getFieldsValues(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, int)
     */
    public Object[][] getFieldsValues(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count) 
    {
        return rdrItf.getFieldsValues(testName, bladeId, eventTypeLabel, XFieldLabel, YFieldLabel, start, end, count);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getMaxValues(java.lang.Object[][], int, int)
     */
    public Object[][] getMaxValues(Object[][] values, int timeWindow, int step) 
    {
        return scItf.getMaxValues(values, timeWindow, step);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getMeans(java.lang.Object[][], int, int)
     */
    public Object[][] getMeans(Object[][] values, int timeWindow, int step) 
    {
        return scItf.getMeans(values, timeWindow, step);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getMinValues(java.lang.Object[][], int, int)
     */
    public Object[][] getMinValues(Object[][] values, int timeWindow, int step) 
    {
        return scItf.getMinValues(values, timeWindow, step);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getStandardDeviations(java.lang.Object[][], int, int)
     */
    public Object[][] getStandardDeviations(Object[][] values, int timeWindow, int step) 
    {
        return scItf.getStandardDeviations(values, timeWindow, step);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getStatistics(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public Map getStatistics(String testName, String bladeId, String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count) 
    {
        return scItf.getStatistics(testName, bladeId, eventTypeLabel, XFieldLabel, YFieldLabel, start, end, count);
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getTestsNames()
     */
    public String[] getTestsNames() {
        return rdrItf.getTestsNames();
    }


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getMaxTimeFor(java.lang.String, java.lang.String, java.lang.String)
     */
    public long getMaxTimeFor(String testName, String bladeId) 
    {
        return rdrItf.getMaxTimeFor(testName, bladeId);
        
    }
    
    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.DataSupplier#getTestPlan(java.lang.String, java.lang.String)
     */
	public BladeDescriptor[] getTestPlan(String testName, BladeFilter filter) {
		try {
			return rdrItf.getTestPlan(testName, filter);
		} catch (Exception e) {
			// TODO: handle exception
		}
		return null;
		
	}
    
	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.DataSupplier#getBladeEvents(java.lang.String, java.lang.String, java.lang.String, org.ow2.clif.storage.api.EventFilter, long, int)
	 */
	public BladeEvent[] getBladeEvents(String testName,	String bladeId, String eventTypeLabel,EventFilter filter, long fromIndex,int count)throws ClifException{
		return rdrItf.getBladeEvents(testName, bladeId, eventTypeLabel, filter, fromIndex, count);
	}
    
	
	/* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.DataSupplier#getBladeEventsValues(org.ow2.clif.storage.api.BladeEvent[], java.lang.String, java.lang.String)
	 */
	public Object[][] getBladeEventsValues(BladeEvent[] bladeEventList, String XFieldLabel, String YFieldLabel){
		 return rdrItf.getBladeEventsValues(bladeEventList, XFieldLabel, YFieldLabel);
	}
	   
	 /* (non-Javadoc)
	 * @see org.ow2.clif.analyze.api.graph.DataSupplier#getBladeDescriptor(java.lang.String, org.ow2.clif.storage.api.BladeFilter)
	 */
	public BladeDescriptor[] getBladeDescriptor(String testName, BladeFilter filter){
		 return rdrItf.getBladeDescriptor(testName, filter);
	 }
	
    /////////////////////////////////
    // interface BindingController //
    /////////////////////////////////


    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(RawDataRead.RAW_DATA_READ))
        {
            rdrItf = (RawDataRead) serverItf;
        } 
        else if (clientItfName.equals(StatisticCollector.STATISTIC_COLLECTOR))
        {
            scItf = (StatisticCollector) serverItf;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() 
    {
        return itfList;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
    {
        if (clientItfName.equals(RawDataRead.RAW_DATA_READ))
        {
            return rdrItf;
        }
        else if (clientItfName.equals(StatisticCollector.STATISTIC_COLLECTOR))
        {
            return scItf;
        }
        else
        {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(RawDataRead.RAW_DATA_READ))
        {
            rdrItf=null;
        }
        else if (clientItfName.equals(StatisticCollector.STATISTIC_COLLECTOR))
        {
            scItf=null;
        }
    }



}
