/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import org.apache.commons.math.stat.StatUtils;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.clif.analyze.api.graph.DataSupplier;
import org.ow2.clif.analyze.api.graph.datasource.StatisticCollector;

/**
 * The Class StatisticImpl.
 */
public class StatisticImpl implements BindingController, StatisticCollector {

    static private String[] itfList = new String[] { 
        DataSupplier.DATA_SUPPLIER };


    // field for client binding (access to stored data)
    /** The data supplier interface. */
    private DataSupplier dsItf;
    
    /**
     * Instantiates a new statistic impl.
     */
    public StatisticImpl(){
        
    }
    
    /////////////////////////////////
    // interface BindingController //
    /////////////////////////////////


    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = (DataSupplier)serverItf;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() 
    {
        return itfList;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            return dsItf;
        }
        else
        {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(DataSupplier.DATA_SUPPLIER))
        {
            dsItf = null;
        }
    }

    
    //////////////////////////////////
    // interface StatisticCollector //
    //////////////////////////////////


    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.StatisticCollector#getMaxValues(java.lang.Object[][], int, int)
     */
    public Object[][] getMaxValues(Object[][] values, int timeWindow, int step) 
    {
        Object[][] maxValues = null;
        double[] vals = toDouble(values[1]);
        Long[] splitTime = split(values[0], timeWindow, step);
        maxValues = new Object[2][splitTime.length];
        
        for (int i = 0; i < splitTime.length; i++) {
            maxValues[0][i] = timeWindow + i * step;
            if (i == splitTime.length-1)
            {
                maxValues[1][i] = StatUtils.max(vals, splitTime[i].intValue(), vals.length-splitTime[i].intValue());
            }
            else
            {
                maxValues[1][i] = StatUtils.max(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue()));
            }
        }
        
        return maxValues;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.StatisticCollector#getMeans(java.lang.Object[][], int, int)
     */
    public Object[][] getMeans(Object[][] values, int timeWindow, int step) 
    {
        Object[][] meanValues = null;
        double[] vals = toDouble(values[1]);
        Long[] splitTime = split(values[0], timeWindow, step);
        
        meanValues = new Object[2][splitTime.length];
        
        for (int i = 0; i < splitTime.length; i++) {
            meanValues[0][i] = timeWindow + i * step;
            if (i == splitTime.length-1)
            {
                meanValues[1][i] = StatUtils.mean(vals, splitTime[i].intValue(), vals.length-splitTime[i].intValue());
            }
            else
            {
                meanValues[1][i] = StatUtils.mean(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue()));
            }
        }
        
        return meanValues;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.StatisticCollector#getMinValues(java.lang.Object[][], int, int)
     */
    public Object[][] getMinValues(Object[][] values, int timeWindow, int step) 
    {
        Object[][] minValues = null;
        double[] vals = toDouble(values[1]);
        Long[] splitTime = split(values[0], timeWindow, step);
        
        minValues = new Object[2][splitTime.length];
        
        for (int i = 0; i < splitTime.length; i++) {
            minValues[0][i] = timeWindow + i * step;
            if (i == splitTime.length-1)
            {
                minValues[1][i] = StatUtils.min(vals, splitTime[i].intValue(), vals.length-splitTime[i].intValue());
            }
            else
            {
                minValues[1][i] = StatUtils.min(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue()));
            }
        }
        
        return minValues;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.StatisticCollector#getStandardDeviations(java.lang.Object[][], int, int)
     */
    public Object[][] getStandardDeviations(Object[][] values, int timeWindow, int step) 
    {
        Object[][] stdDevValues = null;
        double[] vals = toDouble(values[1]);
        Long[] splitTime = split(values[0], timeWindow, step);
        
        stdDevValues = new Object[2][splitTime.length];
        
        for (int i = 0; i < splitTime.length; i++) {
            stdDevValues[0][i] = timeWindow + i * step;
            if (i == splitTime.length-1)
            {
                stdDevValues[1][i] = Math.sqrt(StatUtils.variance(vals, splitTime[i].intValue(), (vals.length-splitTime[i].intValue())));
            }
            else
            {
                stdDevValues[1][i] = Math.sqrt(StatUtils.variance(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue())));
            }
        }
        
        for (int i = 0; i < splitTime.length-1; i++) {
            stdDevValues[0][i] = timeWindow + i * step;
            stdDevValues[1][i] = Math.sqrt(StatUtils.variance(vals, splitTime[i].intValue(), (splitTime[i+1].intValue()-splitTime[i].intValue())));
        }
        
        return stdDevValues;
    }

    /* (non-Javadoc)
     * @see org.ow2.clif.analyze.api.graph.datasource.StatisticCollector#getStatistics(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
     */
    public Map getStatistics(String testName, String bladeId,
            String eventTypeLabel, String XFieldLabel, String YFieldLabel, int start, int end, int count) {
        Map<String, Double> stats = new HashMap<String, Double>();
        Object[][] values = dsItf.getFieldsValues(testName, bladeId, eventTypeLabel, XFieldLabel, YFieldLabel, start, end, count);
        double[] vals = toDouble(values[1]); 
        
        stats.put("max", StatUtils.max(vals));
        stats.put("min", StatUtils.min(vals));
        stats.put("mean", StatUtils.mean(vals));
        stats.put("variance", StatUtils.variance(vals));
        stats.put("standard deviation", Math.sqrt(stats.get("variance")));
        
        return stats;
    }

    
    ///////////////////////
    // private functions //
    ///////////////////////
    

    /**
     * Split.
     * 
     * @param values the values
     * @param timeWindow the time window
     * @param step the step
     * 
     * @return the long[] 
     */
    private Long[] split(Object[] values, int timeWindow, int step){
        long prev=0;
        int k=0;
        int i = 0;
        SortedSet<Long> res = new TreeSet<Long>();
        
        while(i<values.length)
        {
            while( (i<values.length) && ((((Number)values[i]).longValue())<(k*step)+timeWindow) )
            {
                prev = i;
                i++;
            }
            res.add(prev);
            k++;
            prev = i;
            i++;
        }
        return res.toArray(new Long[0]);
    }
    
    
    /**
     * To double.
     * 
     * @param values the values
     * 
     * @return the double[]
     */
    private double[] toDouble(Object[] values)
    {
        double[] vals = new double[values.length];

        if (values.length>0){
            if(values[0] instanceof Number)
            {
                for (int i = 0; i < values.length && values[i]!=null; i++) {
                    vals[i] = ((Number)values[i]).doubleValue();
                }
            }
            else if (values[0] instanceof Boolean)
            {
                for (int i = 0; i < values.length && values[i]!=null; i++) {
                    vals[i] = ((Boolean)values[i]).booleanValue()?1.0:0.0;
                }
            }
            else
            {
                for (int i = 0; i < values.length && values[i]!=null; i++) {
                    vals[i] = 1.0;
                }
            }
        }
        return vals;
    }
}
