/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2005, 2011 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.console.lib;

import java.io.OutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Properties;

/**
 * Utility class for writing a test plan as a property file.
 * @author Bruno Dillenseger
 */
public class TestPlanWriter
{
	static public void write2prop(OutputStream out, Map<String,ClifDeployDefinition> testPlan)
		throws IOException
	{
		TestPlanWriter writer = new TestPlanWriter(out);
		writer.write2prop(testPlan);
		writer.close();
	}


	OutputStream out;


	public TestPlanWriter(OutputStream out)
	{
		this.out = out;
	}


	public void close()
		throws IOException
	{
		out.close();
	}


	public void flush()
		throws IOException
	{
		out.flush();
	}


	public void write2prop(Map<String,ClifDeployDefinition> testPlan)
		throws IOException
	{
		int n = 0;
		Properties props = new Properties();
		for (Map.Entry<String,ClifDeployDefinition> entry : testPlan.entrySet())
		{
			String prefix = TestPlanReader.BLADE_PROP + "." + n++ + ".";
			props.setProperty(prefix + TestPlanReader.ID_PROP, entry.getKey());
			ClifDeployDefinition def = entry.getValue();
			if (def.isProbe())
			{
				props.setProperty(
					prefix + TestPlanReader.PROBE_PROP,
						def.getContext().get("insert"));
			}
			else
			{
				props.setProperty(
					prefix + TestPlanReader.INJECTOR_PROP,
						def.getContext().get("insert"));
			}
			props.setProperty(prefix + TestPlanReader.SERVER_PROP, def.getServerName());
			props.setProperty(prefix + TestPlanReader.ARGUMENT_PROP, def.getArgument());
			props.setProperty(prefix + TestPlanReader.COMMENT_PROP, def.getComment());
		}
		props.store(out, "CLIF test plan");
	}
}
