/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2005, 2006 France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF $Name: not supported by cvs2svn $
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.console.lib.batch;

import org.ow2.clif.deploy.ClifAppFacade;
import org.ow2.clif.deploy.ClifRegistry;
import org.ow2.clif.util.ExecutionContext;

/**
 * Batch command to undeploy a given testplan and unbind it from the Registry.
 * 
 * @author Joan Chaumont
 * @author Bruno Dillenseger
 */
public class QuitCmd
{
	/**
	 * Undeploys a given testplan and unbinds it from the Registry
	 * @param args args[0] is the name of the test plan to undeploy.
	 */
	public static void main(String[] args)
	{
		if (args.length != 1)
		{
			BatchUtil.usage("argument expected: <name of test plan to undeploy>");
		}
		if (System.getSecurityManager() == null)
		{
			System.setSecurityManager(new SecurityManager());
		}
		ExecutionContext.init("./");
		System.exit(run(args[0]));
	}
	
	
	/**
	 * Undeploys a given testplan and unbinds it from the Registry
	 * @param testPlan the name of the test plan to undeploy.
	 * @return execution status code (@see BatchUtil)
	 */
	static public int run(String testPlan)
	{
		try
		{
			ClifRegistry clifReg = new ClifRegistry(false);
			ClifAppFacade clifApp = BatchUtil.getClifAppFacade(testPlan);
			if (clifApp == null)
			{
				System.err.println("Unknown test plan");
				return BatchUtil.ERR_DEPLOY;
			}
			clifApp.deploy(null, clifReg);
			return BatchUtil.SUCCESS;
		}
		catch (Exception ex)
		{
			System.err.println("Execution problem while undeploying");
			ex.printStackTrace(System.err);
			return BatchUtil.ERR_EXEC;
		}
	}
}
