/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2005, 2011 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.storage.api;

import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.util.StringSplitter;


/**
 * Super class for events generated by probes.
 * @author Bruno Dillenseger
 */
abstract public class ProbeEvent extends AbstractEvent
{
	private static final long serialVersionUID = -3642361469292739752L;


	static protected BladeEvent fillEvent(String separator, String line, ProbeEvent event)
		throws ClifException
	{
		try
		{
			String[] values = StringSplitter.split(line, separator);
			event.values = new long[values.length - 1];
			event.date = Long.parseLong(values[0]);
			for (int i=0 ; i<event.values.length ; ++i)
			{
				event.values[i] = Long.parseLong(values[1+i]);
			}
			return event;
		}
		catch (Exception ex)
		{
			throw new ClifException(
				"Could not make an instance of " + event.getClass() +" from " + line,
				ex);
		}
	};

	public long[] values;


	protected ProbeEvent()
	{
		super(-1);
	}


	public ProbeEvent(long date, long[] values)
	{
		super(date);
		this.values = values;
	}


	public String toString()
	{
		return toString(0, DEFAULT_SEPARATOR);
	}


	//////////////////////////
	// BladeEvent interface //
	//////////////////////////


	public String toString(long originDate, String separator)
	{
		String str = String.valueOf(date - originDate);
		for (int i=0 ; i< values.length ; ++i)
		{
			str += separator + values[i];
		}
		return str;
	}


	public Object getFieldValue(String fieldLabel)
	{
		if (fieldLabel != null)
		{
			String[] labels = getFieldLabels();
			if (fieldLabel.equals(labels[0]))
			{
				return new Long(date);
			}
			else
			{
				for (int i=1 ; i<labels.length ; ++i)
				{
					if (fieldLabel.equals(labels[i]))
					{
						return new Long(values[i-1]);
					}
				}
			}
		}
		return null;
	}
}
