/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.graph.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.AbstractCellEditor;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;


/**
 * The Class AbstractCellEditorColor.
 */
public class AbstractCellEditorColor extends AbstractCellEditor implements TableCellEditor {
	
	/** The my color label. */
	private JLabel myColorLabel = new JLabel(" ");
	
	/** The data quick view container. */
	private DataQuickViewContainer dataQuickViewContainer = null;
	
		
	/**
	 * Instantiates a new abstract cell editor color.
	 * 
	 * @param arg0 the Data Quick View Container
	 */
	public AbstractCellEditorColor(DataQuickViewContainer arg0) {
		this.dataQuickViewContainer = arg0;
		
		myColorLabel.setOpaque(true);
		
		myColorLabel.addMouseListener(new MouseListener(){
			public void mouseClicked(MouseEvent e) {
				Color colorChoice = JColorChooser.showDialog(null, "Choose chart line color.", myColorLabel.getBackground());
				if(colorChoice != null) {
					myColorLabel.setBackground( colorChoice );
					dataQuickViewContainer.setLineChartColor( colorChoice );
					AbstractCellEditorColor.this.fireEditingStopped();
				}
			}

			public void mouseEntered(MouseEvent e) {
				// nothing to do
			}

			public void mouseExited(MouseEvent e) {
				// nothing to do
			}

			public void mousePressed(MouseEvent e) {
				// nothing to do
			}

			public void mouseReleased(MouseEvent e) {
				// nothing to do
			}
			
		});		
	}
	
	/**
	 * Instantiates a new abstract cell editor color.
	 */
	public AbstractCellEditorColor() {		
	}
		
	
	/* (non-Javadoc)
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 */
	public Object getCellEditorValue() {
		Color color = myColorLabel.getBackground();
		String strColor = new String();
		
		strColor = color.getRed() + ":" + color.getGreen() + ":" + color.getBlue();

		return strColor;
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	public Component getTableCellEditorComponent(JTable arg0, Object arg1,
			boolean arg2, int arg3, int arg4) {
		if(arg1 != null) {
			int r = Integer.parseInt( ((String[]) ((String) arg1).split(":"))[0] );
			int g = Integer.parseInt( ((String[]) ((String) arg1).split(":"))[1] );
			int b = Integer.parseInt( ((String[]) ((String) arg1).split(":"))[2] );
			myColorLabel.setBackground(new Color(r, g, b));
		}
		
		return myColorLabel;
	}
}
