/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.graph.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuBar;

import org.ow2.clif.analyze.lib.graph.util.SaveFileFilter;

/**
 * 
 * @author Jordan BRUNIER, Gregory CALONNIER, Olivier LIU
 *
 */
public class DataQuickViewMenu extends JMenuBar{
  
	//Parent
	private static final long serialVersionUID = -5813576160673227028L;

//    static private SwingGUI globalGUI;
    private SwingGUI globalGUI;
    
    //Save as File Chooser
    private JFileChooser saveAsDialog = new JFileChooser();
    
    //File Filter
    private SaveFileFilter saveFileFilter = new SaveFileFilter(); 
    
    private JFileChooser saveRoutineAsDialog = new JFileChooser();
    private SaveFileFilter saveRoutine = new SaveFileFilter();
    
    private String imageType = "PNG";
    
    private SimpleDataset dataset;
    
    //Moving Statistic Preferences Dialog
    private DialogMovingStatisticPreferences movingStatPrefDialog;
    
    
    /** Creates new form DataQuickViewMenu */
    public DataQuickViewMenu() {
        initComponents();
        myInitComponents();
    }

    /**
     * Instantiates a new data quick view menu.
     * 
     * @param _globalGUI the _global GUI
     */
    public DataQuickViewMenu(SwingGUI _globalGUI) {
    	this();
    	this.globalGUI = _globalGUI;
    }
    /** End Constructors */

    
    //------------------------ Private Methods ----------------------//    
    private void myInitComponents() {    	
    	//Init save dialog
    	saveAsDialog.setDialogType(JFileChooser.SAVE_DIALOG);
    	saveAsDialog.setApproveButtonText("Save");
    }
    
    //------------------------ Public Methods ----------------------//
    /**
     * Update moving statistic preferences.
     * 
     * @param timeWindow the time window
     * @param step the step
     */
    public void updateMovingStatPreferences(int timeWindow, int step) {
    	this.globalGUI.setTimeWindow(timeWindow);
    	this.globalGUI.setStep(step);
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
    	bgImageType = new ButtonGroup();
        jmiExportToHTML = new javax.swing.JMenuItem();
        jmiExportToXML = new javax.swing.JMenuItem();
        jmiExportToText = new javax.swing.JMenuItem();
        jsFile = new javax.swing.JSeparator();        
        jrbmiPNG = new javax.swing.JRadioButtonMenuItem();
        jrbmiJPG = new javax.swing.JRadioButtonMenuItem();
        jrbmiSVG = new javax.swing.JRadioButtonMenuItem();
        jmiTimeWindowMoving = new javax.swing.JMenuItem();
        jmFile = new javax.swing.JMenu();
        jmExport = new javax.swing.JMenu();
        jmPref = new javax.swing.JMenu();
        jmPrefImg = new javax.swing.JMenu();
        jmDataset = new javax.swing.JMenu();
        jmiSimpleDataset = new javax.swing.JMenuItem();
        jmiMultipleDataset = new javax.swing.JMenuItem();
        jmiAggregateDataset = new javax.swing.JMenuItem();
        jmRoutine = new javax.swing.JMenu();
        jmiSaveRoutine = new javax.swing.JMenuItem();
        jmiSaveRoutine.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	jmiSaveRoutineActionPerformed(evt);
            }
        });
        jmiOpenRoutine = new javax.swing.JMenuItem();
        jmiOpenRoutine.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	jmiOpenRoutineActionPerformed(evt);
            }
        });
    	
        jmFile.setText("File");
        jmFile.setFont(new java.awt.Font("Dialog", 0, 12));
        jmExport.setText("Export");
        jmExport.setFont(new java.awt.Font("Dialog", 0, 12));
        jmRoutine.setText("Routine");
        jmRoutine.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiSaveRoutine.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiSaveRoutine.setText("Save routine");
        jmiOpenRoutine.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiOpenRoutine.setText("Open routine");
        
        jmiExportToHTML.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiExportToHTML.setText("To HTML...");
        jmiExportToHTML.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiExportToHTMLActionPerformed(evt);
            }
        });

        jmExport.add(jmiExportToHTML);

        jmiExportToXML.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiExportToXML.setText("To XML...");
        jmiExportToXML.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiExportToXMLActionPerformed(evt);
            }
        });

        jmExport.add(jmiExportToXML);

        jmiExportToText.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiExportToText.setText("To Text File...");
        jmiExportToText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiExportToTextActionPerformed(evt);
            }
        });

        jmExport.add(jmiExportToText);

        jmDataset.setText("New Dataset");
        jmDataset.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiSimpleDataset.setText("Simple Dataset");
        jmiSimpleDataset.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiSimpleDataset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	jmiSimpleDatasetActionPerformed(evt);
            }
        });
        
        jmDataset.add(jmiSimpleDataset);
        
        jmiMultipleDataset.setText("Multiple Dataset");
        jmiMultipleDataset.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiMultipleDataset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiMultipleDatasetActionPerformed(evt);
            }
        });

        jmDataset.add(jmiMultipleDataset);

        jmiAggregateDataset.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiAggregateDataset.setText("Aggregate Dataset");
        jmiAggregateDataset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiAggregateDatasetActionPerformed(evt);
            }
        });

        jmDataset.add(jmiAggregateDataset);
     
        jmFile.add(jmDataset);      
        jmFile.add(jmExport);
        jmFile.add(jmRoutine);
        jmFile.add(jsFile);
        
        jmRoutine.add(jmiSaveRoutine);
        jmRoutine.add(jmiOpenRoutine);

        this.add(jmFile);

        jmPref.setText("Preferences");
        jmPref.setFont(new java.awt.Font("Dialog", 0, 12));
        jmPrefImg.setText("Export Chart as");
        jmPrefImg.setFont(new java.awt.Font("Dialog", 0, 12));
        bgImageType.add(jrbmiPNG);
        jrbmiPNG.setFont(new java.awt.Font("Dialog", 0, 12));
        jrbmiPNG.setSelected(true);
        jrbmiPNG.setText("PNG");
        jrbmiPNG.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				imageType = "PNG";
			}        	
        });
        jmPrefImg.add(jrbmiPNG);

        bgImageType.add(jrbmiJPG);
        jrbmiJPG.setFont(new java.awt.Font("Dialog", 0, 12));
        jrbmiJPG.setText("JPG");
        jrbmiJPG.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				imageType = "JPG";
			}        	
        });
        jmPrefImg.add(jrbmiJPG);

        bgImageType.add(jrbmiSVG);
        jrbmiSVG.setFont(new java.awt.Font("Dialog", 0, 12));
        jrbmiSVG.setText("SVG");
        jrbmiSVG.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				imageType = "SVG";
			}        	
        });
        jmPrefImg.add(jrbmiSVG);

        jmPref.add(jmPrefImg);

        jmiTimeWindowMoving.setFont(new java.awt.Font("Dialog", 0, 12));
        jmiTimeWindowMoving.setText("Moving Statistic...");
        jmiTimeWindowMoving.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jmiTimeWindowMovingActionPerformed(evt);
            }
        });

        jmPref.add(jmiTimeWindowMoving);


        
        this.add(jmPref);
    }
    
	//add a simple dataset
    private void jmiSimpleDatasetActionPerformed(ActionEvent evt) {
    	dataset = new SimpleDataset(this.globalGUI, true, 0);
    	JFrame jSimpleFrame = new JFrame();
    	jSimpleFrame = dataset;
    	jSimpleFrame.getContentPane().repaint();
    	jSimpleFrame.pack();
    	jSimpleFrame.setVisible(true);
    }

    //add a multiple dataset
    protected void jmiMultipleDatasetActionPerformed(ActionEvent evt) {
    	dataset = new SimpleDataset(this.globalGUI, true, 1);
    	JFrame jSimpleFrame = new JFrame();
    	jSimpleFrame = dataset;
    	jSimpleFrame.getContentPane().repaint();
    	jSimpleFrame.pack();
    	jSimpleFrame.setVisible(true);
    }

    //add an aggregate dataset
    protected void jmiAggregateDatasetActionPerformed(ActionEvent evt) {
    	dataset = new SimpleDataset(this.globalGUI, true, 2);
    	JFrame jSimpleFrame = new JFrame();
    	jSimpleFrame = dataset;
    	jSimpleFrame.getContentPane().repaint();
    	jSimpleFrame.pack();
    	jSimpleFrame.setVisible(true);
    }


	private void jmiTimeWindowMovingActionPerformed(java.awt.event.ActionEvent evt) {
    	this.movingStatPrefDialog = new DialogMovingStatisticPreferences(this, 
    			this.globalGUI.getTimeWindow(), 
    			this.globalGUI.getStep());
    	this.movingStatPrefDialog.setVisible(true);
    }

    protected void jmiOpenRoutineActionPerformed(ActionEvent evt) {

    	
	}

	protected void jmiSaveRoutineActionPerformed(ActionEvent evt) {
    	saveRoutine.setDescription("XML files");
    	saveRoutine.setExtensions(new String[]{".xml"});
    	
    	saveRoutineAsDialog.setDialogTitle("Save Routine...");
    	saveRoutineAsDialog.setFileFilter(saveRoutine);
    	
    	saveRoutineAsDialog.showSaveDialog(this.globalGUI.getInternalFrame());
    	
    	if(saveRoutineAsDialog.getSelectedFile() != null) {
			this.globalGUI.saveRoutineAsXML( 
					saveRoutineAsDialog.getSelectedFile().getPath(),
					this.globalGUI.getStatisticMap(),
    				this.globalGUI.getMapTreeComment(),
    				this.globalGUI.getJFreeChartMap(),
	    			this.imageType, 
	    			this.globalGUI.getPageIdAndDatasetIds(),
	    			this.globalGUI.getChartDataList(),
	    			this.globalGUI.getDatasetIdAndGraphOptions()
	    			 );
    	}
	}
    
    private void jmiExportToXMLActionPerformed(java.awt.event.ActionEvent evt) {
    	//FileFilter
    	saveFileFilter.setDescription("XML files");
    	saveFileFilter.setExtensions(new String[]{".xml"});
    	
    	saveAsDialog.setDialogTitle("Save to XML...");
    	saveAsDialog.setFileFilter(saveFileFilter);
    	
    	saveAsDialog.showSaveDialog(this.globalGUI.getInternalFrame());
    	
    	if(saveAsDialog.getSelectedFile() != null) {
			this.globalGUI.exportToXML( 
					saveAsDialog.getSelectedFile().getPath(),
					this.globalGUI.getStatisticMap(),
    				this.globalGUI.getMapTreeComment(),
    				this.globalGUI.getJFreeChartMap(),
	    			this.imageType, 
	    			this.globalGUI.getPageIdAndDatasetIds(),
	    			this.globalGUI.getChartDataList(),
	    			this.globalGUI.getDatasetIdAndGraphOptions()
	    			 );
    	}
    }

    
   private void jmiExportToHTMLActionPerformed(java.awt.event.ActionEvent evt) {
    	//FileFilter
    	saveFileFilter.setDescription("HTML files");
    	saveFileFilter.setExtensions(new String[]{".html", ".htm"});
    	
    	saveAsDialog.setDialogTitle("Save to HTML...");
    	saveAsDialog.setFileFilter(saveFileFilter);
    	
    	saveAsDialog.showSaveDialog(this.globalGUI.getInternalFrame());
    	
    	if(saveAsDialog.getSelectedFile() != null) {
//	    	this.globalGUI.exportToHTML( 							/// TPS old method: does not work
//    				saveAsDialog.getSelectedFile().getPath() + ".old",
//	    			this.imageType, 
//	    			this.globalGUI.getComment(), 
//	    			this.globalGUI.getStatistics(),
//					this.globalGUI.getChartTitle() );

	    	this.globalGUI.exportToHTML( 
					saveAsDialog.getSelectedFile().getPath(),
					this.globalGUI.getStatisticMap(),
    				this.globalGUI.getMapTreeComment(),
    				this.globalGUI.getJFreeChartMap(),
	    			this.imageType, 
	    			this.globalGUI.getPageIdAndDatasetIds(),
	    			this.globalGUI.getChartDataList(),
	    			this.globalGUI.getDatasetIdAndGraphOptions()
	    			);
    	}
    }

    private void jmiExportToTextActionPerformed(java.awt.event.ActionEvent evt) {
    	//FileFilter
    	saveFileFilter.setDescription("Text files");
    	saveFileFilter.setExtensions(new String[]{"."});

    	saveAsDialog.setDialogTitle("Save to Text files...");
    	saveAsDialog.setFileFilter(saveFileFilter);

    	saveAsDialog.showSaveDialog(this.globalGUI.getInternalFrame());

    	if(saveAsDialog.getSelectedFile() != null) {
    		this.globalGUI.exportToText(
    				saveAsDialog.getSelectedFile().getPath(),
    				this.globalGUI.getStatisticMap(),
    				this.globalGUI.getMapTreeComment(),
    				this.globalGUI.getJFreeChartMap(),
    				this.imageType);
    	}
    }
    
  
    // GUI variables declaration   
    private javax.swing.ButtonGroup bgImageType;
    private javax.swing.JMenu jmExport;
    private javax.swing.JMenu jmFile;
    private javax.swing.JMenu jmRoutine;
    private javax.swing.JMenu jmPref;
    private javax.swing.JMenu jmPrefImg;
    private javax.swing.JMenuItem jmiExportToHTML;
    private javax.swing.JMenuItem jmiExportToText;
    private javax.swing.JMenuItem jmiExportToXML;
    private javax.swing.JMenuItem jmiSaveRoutine;
    private javax.swing.JMenuItem jmiOpenRoutine;
    private javax.swing.JMenuItem jmiTimeWindowMoving;
    private javax.swing.JRadioButtonMenuItem jrbmiJPG;
    private javax.swing.JRadioButtonMenuItem jrbmiPNG;
    private javax.swing.JRadioButtonMenuItem jrbmiSVG;
    private javax.swing.JSeparator jsFile;
    private javax.swing.JMenu jmDataset;
    private javax.swing.JMenuItem jmiSimpleDataset;
    private javax.swing.JMenuItem jmiMultipleDataset;
    private javax.swing.JMenuItem jmiAggregateDataset;
}
