/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF $Name: not supported by cvs2svn $
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.console.lib.gui;


import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JTabbedPane;

import org.ow2.clif.console.lib.ClifDeployDefinition;
import org.ow2.clif.deploy.ClifAppFacade;


/**
 *
 * @author Bruno Dillenseger
 */
public class GuiMonitorPanel
	extends JTabbedPane
	implements ContainerListener
{
	static private final String INJECTOR_LABEL = "injector";


	/**
     * Constructor
	 * @param testPlan
	 * @param clifApp
	 */
	public GuiMonitorPanel(Map testPlan, ClifAppFacade clifApp)
	{
		super();
		Map bladeKinds = new HashMap();
		Iterator iter = testPlan.entrySet().iterator();
		while (iter.hasNext())
		{
			Map.Entry entry = (Map.Entry) iter.next();
			ClifDeployDefinition def = (ClifDeployDefinition)entry.getValue();
			String key;
			if (def.isProbe())
			{
				key = def.getContext().get("insert");
				key = key.substring(0, key.lastIndexOf('.'));
				key = key.substring(1 + key.lastIndexOf('.'));
			}
			else
			{
				key = INJECTOR_LABEL;
			}
			Map blades = (Map)bladeKinds.get(key);
			if (blades == null)
			{
				blades = new HashMap();
				bladeKinds.put(key, blades);
			}
			blades.put(entry.getKey(), entry.getValue());
		}
		iter = bladeKinds.entrySet().iterator();
		while (iter.hasNext())
		{
			Map.Entry entry = (Map.Entry)iter.next();
			String arbitraryBladeId = (String)((Map)entry.getValue()).keySet().iterator().next();
			GuiMonitorCard monitorCard = new GuiMonitorCard(
				(Map)entry.getValue(),
				clifApp.getStatLabels(arbitraryBladeId),
				clifApp);
			this.addContainerListener(monitorCard);
			add(monitorCard, entry.getKey());
		}
	}


	/////////////////////////////////
	// interface ContainerListener //
	/////////////////////////////////


	public void componentAdded(ContainerEvent ev)
	{
	}


	public void componentRemoved(ContainerEvent ev)
	{
		removeAll();
	}
}
