package org.ow2.clif.loadtest;

import hudson.model.AbstractBuild;
import org.kohsuke.stapler.StaplerRequest;
import org.kohsuke.stapler.StaplerResponse;
import org.ow2.clif.loadtest.model.LoadTestReport;
import org.ow2.clif.loadtest.parser.clif.chart.CallChart;
import org.ow2.clif.loadtest.parser.clif.chart.FixedSliceNumberDistributionChart;
import org.ow2.clif.loadtest.parser.clif.chart.FixedSliceSizeDistributionChart;
import org.ow2.clif.loadtest.parser.clif.chart.QuantileDistributionChart;

import java.io.IOException;
import java.io.PrintStream;
import java.util.Properties;

/**
 * Action used for LoadTest report on build level.
 *
 * @author Julien Coste
 */
public class LoadTestBuildAction
    extends AbstractLoadTestAction
{
    private final AbstractBuild<?, ?> build;

    private LoadTestReport report;

    private transient Properties alias;

    public LoadTestBuildAction( AbstractBuild<?, ?> build, LoadTestReport report, LoadTestPublisher publisher,
                                PrintStream logger )
    {
        this.build = build;
        this.report = report;
        logger.println( "Created LoadTest results" );
    }

    public void doCallChart( StaplerRequest request, StaplerResponse response )
        throws IOException
    {

        LoadTestGraphParam params = new LoadTestGraphParam();
        request.bindParameters( params );

        CallChart chart = new CallChart( params.getTestPlan(), params.getBladeId(), params.getLabel() );
        chart.doPng( build.getRootDir(), request, response );
    }

    public void doFixedSliceNumberDistributionChart( StaplerRequest request, StaplerResponse response )
        throws IOException
    {

        LoadTestGraphParam params = new LoadTestGraphParam();
        request.bindParameters( params );

        FixedSliceNumberDistributionChart chart =
            new FixedSliceNumberDistributionChart( params.getTestPlan(), params.getBladeId(), params.getLabel(), -1 );
        chart.doPng( build.getRootDir(), request, response );
    }

    public void doFixedSliceSizeDistributionChart( StaplerRequest request, StaplerResponse response )
        throws IOException
    {

        LoadTestGraphParam params = new LoadTestGraphParam();
        request.bindParameters( params );

        FixedSliceSizeDistributionChart chart =
            new FixedSliceSizeDistributionChart( params.getTestPlan(), params.getBladeId(), params.getLabel(), -1 );
        chart.doPng( build.getRootDir(), request, response );
    }

    public void doQuantileDistributionChart( StaplerRequest request, StaplerResponse response )
        throws IOException
    {

        LoadTestGraphParam params = new LoadTestGraphParam();
        request.bindParameters( params );

        QuantileDistributionChart chart =
            new QuantileDistributionChart( params.getTestPlan(), params.getBladeId(), params.getLabel() );
        chart.doPng( build.getRootDir(), request, response );
    }

    public AbstractBuild<?, ?> getBuild()
    {
        return build;
    }

    public LoadTestReport getReport()
    {
        return report;
    }

    @Override
    public String getDisplayName()
    {
        return Messages.BuildAction_DisplayName();
    }
}
