/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest.parser.clif.chart;

import org.jfree.chart.JFreeChart;
import org.kohsuke.stapler.StaplerRequest;
import org.kohsuke.stapler.StaplerResponse;

import javax.imageio.ImageIO;
import javax.servlet.ServletOutputStream;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * @author Julien Coste
 */
public abstract class AbstractChart
{
    protected String chartId;

    protected String testplan;

    protected String bladeId;

    protected String event;

    public AbstractChart( String chartId, String bladeId, String testplan, String event )
    {
        this.chartId = chartId;
        this.bladeId = bladeId;
        this.testplan = testplan;
        this.event = event;
    }

    protected static String hash( String plaintext )
    {
        MessageDigest md;

        try
        {
            md = MessageDigest.getInstance( "SHA-1" ); // SHA-1 generator instance
        }
        catch ( NoSuchAlgorithmException e )
        {
            return "";
        }

        try
        {
            //8859_1 ou UTF-8
            md.update( plaintext.getBytes( "UTF-8" ) ); // Message summary generation
        }
        catch ( UnsupportedEncodingException e )
        {
            return "";
        }

        byte raw[] = md.digest(); // Message summary reception

        String HEX_DIGITS = "0123456789abcdef";
        StringBuffer result = new StringBuffer();
        for ( int i = 0; i < raw.length; i++ )
        {
            byte b = raw[i];
            result.append( HEX_DIGITS.charAt( ( b & 0xF0 ) >> 4 ) );
            result.append( HEX_DIGITS.charAt( ( b & 0x0F ) ) );
        }

        return result.toString();
    }

    private void saveImageFile( File imageFile, BufferedImage bImage )
    {
        try
        {
            OutputStream os = new FileOutputStream( imageFile );
            ImageIO.write( bImage, "PNG", os );
            os.close();
        }
        catch ( IOException e )
        {
            e.printStackTrace();
        }
    }

    private File getImageFile( File rootDir )
    {
        File loadTestImageDir = new File( rootDir, "loadtest" );
        if ( loadTestImageDir.exists() )
        {
            if ( !loadTestImageDir.isDirectory() )
            {
                throw new IllegalStateException( loadTestImageDir.getAbsolutePath() + " is not a directory" );
            }
            if ( !loadTestImageDir.canWrite() )
            {
                throw new IllegalStateException( loadTestImageDir.getAbsolutePath() + " is not writable" );
            }
        }
        else
        {
            if ( !loadTestImageDir.mkdirs() )
            {
                throw new IllegalStateException(
                    "Impossible to create directory " + loadTestImageDir.getAbsolutePath() );
            }
        }

        return new File( loadTestImageDir, getFilename() + ".png" );
    }

    private String getFilename()
    {
        return hash( this.chartId + this.testplan + this.bladeId + this.event );
    }

    public void doPng( File rootDir, StaplerRequest req, StaplerResponse rsp )
        throws IOException
    {

        File imageFile = getImageFile( rootDir );

        try
        {
            BufferedImage bufferedImage = ImageIO.read( imageFile );
            rsp.setContentType( "image/png" );
            ServletOutputStream os = rsp.getOutputStream();
            ImageIO.write( bufferedImage, "PNG", os );
            os.close();
        }
        catch ( Error e )
        {
            /* OpenJDK on ARM produces an error like this in case of headless error
                Caused by: java.lang.Error: Probable fatal error:No fonts found.
                        at sun.font.FontManager.getDefaultPhysicalFont(FontManager.java:1088)
                        at sun.font.FontManager.initialiseDeferredFont(FontManager.java:967)
                        at sun.font.CompositeFont.doDeferredInitialisation(CompositeFont.java:254)
                        at sun.font.CompositeFont.getSlotFont(CompositeFont.java:334)
                        at sun.font.CompositeStrike.getStrikeForSlot(CompositeStrike.java:77)
                        at sun.font.CompositeStrike.getFontMetrics(CompositeStrike.java:93)
                        at sun.font.Font2D.getFontMetrics(Font2D.java:387)
                        at java.awt.Font.defaultLineMetrics(Font.java:2082)
                        at java.awt.Font.getLineMetrics(Font.java:2152)
                        at org.jfree.chart.axis.NumberAxis.estimateMaximumTickLabelHeight(NumberAxis.java:974)
                        at org.jfree.chart.axis.NumberAxis.selectVerticalAutoTickUnit(NumberAxis.java:1104)
                        at org.jfree.chart.axis.NumberAxis.selectAutoTickUnit(NumberAxis.java:1048)
                        at org.jfree.chart.axis.NumberAxis.refreshTicksVertical(NumberAxis.java:1249)
                        at org.jfree.chart.axis.NumberAxis.refreshTicks(NumberAxis.java:1149)
                        at org.jfree.chart.axis.ValueAxis.reserveSpace(ValueAxis.java:788)
                        at org.jfree.chart.plot.CategoryPlot.calculateRangeAxisSpace(CategoryPlot.java:2650)
                        at org.jfree.chart.plot.CategoryPlot.calculateAxisSpace(CategoryPlot.java:2669)
                        at org.jfree.chart.plot.CategoryPlot.draw(CategoryPlot.java:2716)
                        at org.jfree.chart.JFreeChart.draw(JFreeChart.java:1222)
                        at org.jfree.chart.JFreeChart.createBufferedImage(JFreeChart.java:1396)
                        at org.jfree.chart.JFreeChart.createBufferedImage(JFreeChart.java:1376)
                        at org.jfree.chart.JFreeChart.createBufferedImage(JFreeChart.java:1361)
                        at hudson.util.ChartUtil.generateGraph(ChartUtil.java:116)
                        at hudson.util.ChartUtil.generateGraph(ChartUtil.java:99)
                        at hudson.tasks.test.AbstractTestResultAction.doPng(AbstractTestResultAction.java:196)
                        at hudson.tasks.test.TestResultProjectAction.doTrend(TestResultProjectAction.java:97)
                        ... 37 more
             */
            if ( e.getMessage().contains( "Probable fatal error:No fonts found" ) )
            {
                rsp.sendRedirect2( req.getContextPath() + "/images/headless.png" );
                return;
            }
            throw e; // otherwise let the caller deal with it
        }
        catch ( HeadlessException e )
        {
            // not available. send out error message
            rsp.sendRedirect2( req.getContextPath() + "/images/headless.png" );
        }

    }

    public void createChart( File rootDir, int chartWidth, int chartHeight )
    {
        JFreeChart chart = createChart();

        File imageFile = getImageFile( rootDir );
        BufferedImage bImage = chart.createBufferedImage( chartWidth, chartHeight );
        saveImageFile( imageFile, bImage );

    }

    protected abstract JFreeChart createChart();
}
