/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest;

import org.junit.Test;

import static junit.framework.Assert.assertTrue;

/**
 * @author Julien coste
 */
public class ClifBuilderTest {

    @Test
    public void testGetOverridedPropertiesWithDir() {

        ClifBuilder clifBuilder = new ClifBuilder("Clif", "Ant", "", null, "src/perf/MyTestPlan.ctp", "target/report");

        String overridedProperties = clifBuilder.getOverridedProperties();
        System.out.println(overridedProperties);
        assertTrue(overridedProperties.contains("testplan.name=MyTestPlan"));
        assertTrue(overridedProperties.contains("testplan.file=src/perf/MyTestPlan.ctp"));
        assertTrue(overridedProperties.contains("testrun.id=MyTestPlan"));
        assertTrue(overridedProperties.contains("test.report=target/report"));
        assertTrue(overridedProperties.contains("testplan.code.path=src/perf/"));
    }

    @Test
    public void testGetOverridedPropertiesWithoutDir() {

        ClifBuilder clifBuilder = new ClifBuilder("Clif", "Ant", "", null, "MyTestPlan2.ctp", "target");

        String overridedProperties = clifBuilder.getOverridedProperties();
        System.out.println(overridedProperties);
        assertTrue(overridedProperties.contains("testplan.name=MyTestPlan2"));
        assertTrue(overridedProperties.contains("testplan.file=MyTestPlan2.ctp"));
        assertTrue(overridedProperties.contains("testrun.id=MyTestPlan2"));
        assertTrue(overridedProperties.contains("test.report=target"));
        assertTrue(overridedProperties.contains("testplan.code.path=/"));
    }

    @Test
    public void testGetOverridedPropertiesWithProperties() {

        ClifBuilder clifBuilder = new ClifBuilder("Clif", "Ant", "", "dummy=dummy", "MyTestPlan2.ctp", "target");

        String overridedProperties = clifBuilder.getOverridedProperties();
        System.out.println(overridedProperties);
        assertTrue(overridedProperties.contains("testplan.name=MyTestPlan2"));
        assertTrue(overridedProperties.contains("testplan.file=MyTestPlan2.ctp"));
        assertTrue(overridedProperties.contains("testrun.id=MyTestPlan2"));
        assertTrue(overridedProperties.contains("test.report=target"));
        assertTrue(overridedProperties.contains("testplan.code.path=/"));
        assertTrue(overridedProperties.contains("dummy=dummy\n"));
    }
}
