/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.statistics.SimpleHistogramBin;
import org.jfree.data.statistics.SimpleHistogramDataset;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;

import java.util.Random;

/**
 * A demo scatter plot.
 *
 * @author Julien Coste
 */
public class Histo2Demo extends ApplicationFrame {

    /**
     * A demonstration application showing a scatter plot.
     *
     * @param title the frame title.
     */
    public Histo2Demo(String title) {

        super(title);

        SimpleHistogramDataset data = new SimpleHistogramDataset("test");

        double lower = 0;
        int sliceSize = 5;
        do {
            boolean last = (lower + sliceSize >= 100);
            data.addBin(new SimpleHistogramBin(lower, lower + sliceSize, true, last));
            lower += sliceSize;
        } while (lower < 100);

        data.setAdjustForBinSize(false);


        double[] values = new double[1000];
        Random r = new Random();
        for (int i = 0; i < values.length; i++) {
            values[i] = r.nextInt(100);
        }
        data.addObservations(values);
        JFreeChart chart = ChartFactory.createHistogram(
                "Bar chart Demo",
                "X",
                "Y",
                data,
                PlotOrientation.VERTICAL,
                true,
                true,
                false
        );
        System.out.println("ItemCount  = " + data.getItemCount(0));
        for (int i = 0; i < data.getItemCount(0); i++) {
            System.out.print(i + " -> " + data.getStartX(0, i) + " " + data.getEndX(0, i));
            if (i > 0) {
                System.out.println("  " + (data.getStartX(0, i).doubleValue() - data.getStartX(0, i - 1).doubleValue()));
            }
        }

        double tickSize = data.getStartX(0, 1).doubleValue() - data.getStartX(0, 0).doubleValue();
        double rangeStart = data.getStartX(0, 0).doubleValue();
        double rangeEnd = data.getEndX(0, data.getItemCount(0) - 1).doubleValue();

        //NumberAxis domainAxis = (NumberAxis) chart.getXYPlot().setDgetDomainAxis();
        NumberAxis domainAxis = (NumberAxis) new HistogramAxis(data, 0);
        domainAxis.setAutoRangeIncludesZero(false);
        domainAxis.setVerticalTickLabels(true);
        domainAxis.setTickLabelsVisible(true);
        domainAxis.setTickMarksVisible(true);

        domainAxis.setRange(rangeStart, rangeEnd);

        chart.getXYPlot().setDomainAxis(domainAxis);


        chart.getXYPlot().setRangeGridlinesVisible(true);
        chart.getXYPlot().setDomainGridlinesVisible(false);

        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        chartPanel.setVerticalAxisTrace(false);
        chartPanel.setHorizontalAxisTrace(true);
        setContentPane(chartPanel);

    }

    // ****************************************************************************
    // * JFREECHART DEVELOPER GUIDE                                               *
    // * The JFreeChart Developer Guide, written by David Gilbert, is available   *
    // * to purchase from Object Refinery Limited:                                *
    // *                                                                          *
    // * http://www.object-refinery.com/jfreechart/guide.html                     *
    // *                                                                          *
    // * Sales are used to provide funding for the JFreeChart project - please    *
    // * support us so that we can continue developing free software.             *
    // ****************************************************************************

    /**
     * Starting point for the demonstration application.
     *
     * @param args ignored.
     */
    public static void main(String[] args) {

        Histo2Demo demo = new Histo2Demo("Scatter Plot Demo");
        demo.pack();
        RefineryUtilities.centerFrameOnScreen(demo);
        demo.setVisible(true);

    }

}
