/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest;

import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTick;
import org.jfree.chart.axis.Tick;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.TextAnchor;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Julien Coste
 */
public class HistogramAxis extends NumberAxis {

    private IntervalXYDataset data;
    private int serie;
    private NumberFormat formatter = new DecimalFormat("###.##");

    public HistogramAxis(IntervalXYDataset data, int serie) {
        super();
        this.data = data;
        this.serie = serie;
    }

    @Override
    protected List refreshTicksHorizontal(Graphics2D g2, Rectangle2D dataArea, RectangleEdge edge) {
        List result = new ArrayList();

        double currentTickValue = 50.5;
        for (int i = 0; i < data.getItemCount(serie); i++) {
            currentTickValue = data.getStartX(0, i).doubleValue();
            Tick tick = createTick(edge, currentTickValue);
            result.add(tick);
        }
        // Add last tick
        currentTickValue = data.getEndX(0, data.getItemCount(serie) - 1).doubleValue();
        Tick tick = createTick(edge, currentTickValue);
        result.add(tick);
        return result;
    }

    private Tick createTick(RectangleEdge edge, double currentTickValue) {
        String tickLabel;
        tickLabel = formatter.format(currentTickValue);

        TextAnchor anchor = null;
        TextAnchor rotationAnchor = null;
        double angle = 0.0;
        if (isVerticalTickLabels()) {
            anchor = TextAnchor.CENTER_RIGHT;
            rotationAnchor = TextAnchor.CENTER_RIGHT;
            if (edge == RectangleEdge.TOP) {
                angle = Math.PI / 2.0;
            } else {
                angle = -Math.PI / 2.0;
            }
        } else {
            if (edge == RectangleEdge.TOP) {
                anchor = TextAnchor.BOTTOM_CENTER;
                rotationAnchor = TextAnchor.BOTTOM_CENTER;
            } else {
                anchor = TextAnchor.TOP_CENTER;
                rotationAnchor = TextAnchor.TOP_CENTER;
            }
        }
        return new NumberTick(new Double(currentTickValue), tickLabel, anchor,
                rotationAnchor, angle
        );
    }
}
