/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest.parser.clif;

import org.junit.Test;
import org.ow2.clif.storage.api.BladeDescriptor;
import org.ow2.clif.storage.api.TestDescriptor;

import java.util.Date;

import static org.junit.Assert.assertEquals;

/**
 * @author Julien Coste
 */
public class ActionStatInfoTest {
    ActionStatInfo actionStat;


    @Test
    public void testWithoutDataCleanup() throws Exception {

        ParsingContext ctx = createContext("ActionStatInfoTest");
        ctx.setDataCleanup(false);
        actionStat = new ActionStatInfo(ctx);
        ActionStatInfo.MIN_SIZE_OF_STATISTICAL_DATA = 10;

        actionStat.addStat(1, 1);
        actionStat.addStat(2, 1);
        actionStat.addStat(3, 1);
        actionStat.addStat(4, 1);
        actionStat.addStat(5, 0);
        actionStat.addStat(6, 14);
        actionStat.addStat(7, 1);
        actionStat.addStat(8, 1);
        actionStat.addStat(9, 1);
        actionStat.addStat(10, 1);
        actionStat.addStat(11, 1);
        actionStat.addStat(12, 1);


        actionStat.compute();

        assertEquals(12, actionStat.getN());
        assertEquals(0, actionStat.getErrors());
        assertEquals(14, actionStat.getMax(), 0.0001);
        assertEquals(0, actionStat.getMin(), 0.0001);
        assertEquals(2, actionStat.getMean(), 0.0001);
        assertEquals(1, actionStat.getPercentile(50), 0.0001);

    }

    @Test
    public void testWithDataCleanup() throws Exception {

        ParsingContext ctx = createContext("ActionStatInfoTest");
        ctx.setDataCleanup(true);
        ctx.setKeepFactor(1);
        ctx.setKeepPercentage(50);
        actionStat = new ActionStatInfo(ctx);
        ActionStatInfo.MIN_SIZE_OF_STATISTICAL_DATA = 10;

        actionStat.addStat(1, 1);
        actionStat.addStat(2, 1);
        actionStat.addStat(3, 1);
        actionStat.addStat(4, 1);
        actionStat.addStat(5, 0);
        actionStat.addStat(6, 14);
        actionStat.addStat(7, 1);
        actionStat.addStat(8, 1);
        actionStat.addStat(9, 1);
        actionStat.addStat(10, 1);
        actionStat.addStat(11, 1);
        actionStat.addStat(12, 1);

        actionStat.compute();

        assertEquals(10, actionStat.getN());
        assertEquals(0, actionStat.getErrors());
        assertEquals(1, actionStat.getMax(), 0.0001);
        assertEquals(1, actionStat.getMin(), 0.0001);
        assertEquals(1, actionStat.getMean(), 0.0001);
        assertEquals(1, actionStat.getPercentile(50), 0.0001);

    }


    private ParsingContext createContext(final String testName) {
        ParsingContext ctx = new ParsingContext();
        ctx.setTest(new TestDescriptor() {
            public String getName() {
                return testName;
            }

            public Date getDate() {
                return new Date();
            }
        });
        ctx.setBlade(new BladeDescriptor() {
            public String getServerName() {
                return "localhost";
            }

            public String getArgument() {
                return "1000 60";
            }

            public String getClassname() {
                return "ISACRunner";
            }

            public String getComment() {
                return "";
            }

            public String getId() {
                return "2";
            }

            public boolean isProbe() {
                return false;
            }

            public boolean isInjector() {
                return true;
            }

            public String[] getEventTypeLabels() {
                return new String[0];
            }
        });
        ctx.setEventType("Google");
        return ctx;
    }
}
