/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2004, 2008 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Contact: clif@ow2.org
 */
package org.ow2.clif.loadtest.parser.clif;

import org.junit.Test;

import java.util.Arrays;

import static org.junit.Assert.assertTrue;

/**
 * @author Julien Coste
 */
public class DoubleArraySorterTest {
    @Test
    public void testSortLesserThan7() throws Exception {

        double[] values = {21, 12, 53, 24, 35, 66};
        double[] dates = {1, 2, 3, 4, 5, 6};

        double[] expectedValues = {12, 21, 24, 35, 53, 66};
        double[] expectedDates = {2, 1, 4, 5, 3, 6};


        DoubleArraySorter.sort(values, dates);
        assertTrue(Arrays.equals(expectedValues, values));
        assertTrue(Arrays.equals(expectedDates, dates));
    }

    @Test
    public void testSortSimple() throws Exception {

        double[] values = {21};
        double[] dates = {1};

        double[] expectedValues = {21};
        double[] expectedDates = {1};

        DoubleArraySorter.sort(values, dates);
        assertTrue(Arrays.equals(expectedValues, values));
        assertTrue(Arrays.equals(expectedDates, dates));
    }

    @Test
    public void testSortBig() throws Exception {

        double[] values = {11, 92, 23, 84, 35, 76, 47, 68, 59};
        double[] dates = {1, 2, 3, 4, 5, 6, 7, 8, 9};

        double[] expectedValues = {11, 23, 35, 47, 59, 68, 76, 84, 92};
        double[] expectedDates = {1, 3, 5, 7, 9, 8, 6, 4, 2};


        DoubleArraySorter.sort(values, dates);
        assertTrue(Arrays.equals(expectedValues, values));
        assertTrue(Arrays.equals(expectedDates, dates));
    }

    @Test
    public void testSortVeryBig() throws Exception {

        double[] values = new double[50];
        double[] dates = new double[50];
        double[] expectedValues = new double[50];
        double[] expectedDates = new double[50];

        for (int i = 0; i < dates.length; i++) {
            values[i] = dates.length - i;
            dates[i] = dates.length - i;
            expectedValues[i] = i + 1;
            expectedDates[i] = i + 1;
        }

        DoubleArraySorter.sort(values, dates);
        assertTrue(Arrays.equals(expectedValues, values));
        assertTrue(Arrays.equals(expectedDates, dates));
    }
}
