/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.oda.ui;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.text.MessageFormat;

/**
 * ClifDataUI oda driver plugin runtime implementation.
 * @author Grégory CALONNIER
 * @author Jordan BRUNIER
 */
public class ClifDataUiPlugin extends AbstractUIPlugin
{
    
	// The shared instance.
	private static ClifDataUiPlugin plugin;
	// Resource bundle.
	private ResourceBundle resourceBundle;
	
	
	/**
	 * The constructor.
	 */
	public ClifDataUiPlugin( )
	{
		super( );
		plugin = this;
		try
		{
			resourceBundle = ResourceBundle.getBundle( "org.ow2.clif.analyze.lib.oda.ui.messages" );
		}
		catch ( MissingResourceException x )
		{
			resourceBundle = null;
		}
	}

	/**
     * Returns the shared instance.
     * 
     * @return the default
     */
	public static ClifDataUiPlugin getDefault( )
	{
		return plugin;
	}

	/**
     * Returns the string from the plugin's resource bundle, or 'key' if not
     * found.
     * 
     * @param key the key
     * 
     * @return the resource string
     */
	public static String getResourceString( String key )
	{
		ResourceBundle bundle = ClifDataUiPlugin.getDefault( ).getResourceBundle( );
		try
		{
			return ( bundle != null ) ? bundle.getString( key ) : key;
		}
		catch ( MissingResourceException e )
		{
			return key;
		}
	}
	
	/**
     * Returns the string from the Resource bundle, formatted according to the
     * arguments specified.
     * 
     * @param key the key
     * @param arguments the arguments
     * 
     * @return the formatted string
     */
	public static String getFormattedString(String key, Object[] arguments) {
		return MessageFormat.format(getResourceString(key), arguments);
	}

	/**
     * Returns the plugin's resource bundle,.
     * 
     * @return the resource bundle
     */
	public ResourceBundle getResourceBundle( )
	{
		return resourceBundle;
	}
}
