/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Gr�gory CALONNIER
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.analyze.lib.oda.ui.wizards;

import java.util.Properties;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.clif.analyze.api.AnalyzerExternalAccess;
import org.ow2.clif.deploy.ClifAnalyzerAppFacade;
import org.ow2.clif.supervisor.api.ClifException;
import org.ow2.clif.storage.api.TestDescriptor;
import org.ow2.clif.analyze.lib.oda.CommonConstants;

/**
 * Provide an helper to create wizard page to select local CLIF executions.
 * @author Gr�gory CALONNIER
 * @author Jordan BRUNIER
 */
public class ExecutionSelectionPageHelper {
	
	private static final String EMPTY_STRING = "";
	static final String DEFAULT_MESSAGE = "Select local CLIF execution(s)...";
	
	
	// List for selection and selection
	private List executionsList;
	private Text labSelExecution;
	
	private boolean completePage;
	private WizardPage m_wizardPage;
	private PreferencePage m_propertyPage;	
	
	/**
	 * Constructor
	 */
	ExecutionSelectionPageHelper( WizardPage page )
    {
        m_wizardPage = page;
        
        this.completePage = false;
    }
	
	ExecutionSelectionPageHelper( PreferencePage page )
    {
        m_propertyPage = page;
        
        this.completePage = false;
    }

	
	void createCustomControl( Composite parent )
    {
    	this.setMessage( DEFAULT_MESSAGE );
    	Composite container = new Composite(parent, SWT.NULL);		
		GridLayout containerLayout = new GridLayout();
		containerLayout.numColumns = 2;
		containerLayout.verticalSpacing = 9;
		container.setLayout(containerLayout);
		GridData gd;
		
		
		/*AREA : Label + List*/
		Composite leftArea = new Composite(container, SWT.NULL);
		GridLayout layoutLeftArea = new GridLayout();
		leftArea.setLayout(layoutLeftArea);
		layoutLeftArea.numColumns = 1;
		
		Label labExecutionsTitle = new Label(leftArea, SWT.NULL);
		labExecutionsTitle.setText("1. Local available CLIF execution(s) :");

		executionsList = new List(leftArea, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL | SWT.H_SCROLL);
		gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 200;
		gd.widthHint = 200;
		executionsList.setLayoutData(gd);
		executionsList.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addExecution();
			}
		});

		/*Selection LABEL*/
		Composite rightArea = new Composite(container, SWT.NULL);
		GridLayout layoutRightArea = new GridLayout();
		rightArea.setLayout(layoutRightArea);
		layoutRightArea.numColumns = 1;
		
		Label labSelExecutionTitle = new Label(rightArea, SWT.NULL);
		labSelExecutionTitle.setText("2. Selected execution :");
		
		labSelExecution = new Text(rightArea, SWT.SINGLE | SWT.READ_ONLY);
		labSelExecution.setText("");
		gd = new GridData();
		gd.widthHint = 350;
		labSelExecution.setLayoutData(gd);
		labSelExecution.setEditable(false);
		
		initialize();
    }
	
	/**
	 * Add the execution selected from the list to the text field 
	 */
	private void addExecution()
	{
		if(executionsList.getSelectionIndex() != -1)
		{ // There is a selected element
			if(executionsList != null && executionsList.getItem(executionsList.getSelectionIndex()).trim().compareTo("") != 0)
			{ // Item is not empty
				labSelExecution.setText(executionsList.getItem(executionsList.getSelectionIndex()));
				this.completePage = true;	
			}
		} else {
			labSelExecution.setText("");
			this.completePage = false;
		}
		
		if(m_wizardPage != null)
		{
			((ExecutionSelectionWizardPage)m_wizardPage).refresh();
		}
	}
	
	/**
	 * Initialize executions list.
	 */
	private void initialize() {
		// Get all local executions
		AnalyzerExternalAccess analyzer = null;
		ClifAnalyzerAppFacade clifAnalyzerApp = ClifAnalyzerAppFacade.getInstance();
		
		try{	
			Component analyzerComp = clifAnalyzerApp.getComponentByName("analyzer");
			analyzer = (AnalyzerExternalAccess) analyzerComp.getFcInterface(AnalyzerExternalAccess.ANALYZER_EXTERNAL_ACCESS);			
		}
		catch (NoSuchInterfaceException ex)
		{
			System.err.println(ex);
		}		
		TestDescriptor[] executionsDescList = null;
		try{
			executionsDescList = analyzer.getTests(null);
		} catch(ClifException ce) {
			System.err.println(ce);
		}
		
		// Update list with executions
		for(int i = 0; i < executionsDescList.length; i++){
			executionsList.add(executionsDescList[i].getName());
		}
		
		
		// Try to get Propety		
		if(m_wizardPage !=null &&
				((ExecutionSelectionWizardPage)m_wizardPage).getProperties() != null &&
				((ExecutionSelectionWizardPage)m_wizardPage).getProperties().getProperty( CommonConstants.CONN_HOME_DIR_PROP ).trim().compareTo("") != 0)
		{
			String selExecution = ((ExecutionSelectionWizardPage)m_wizardPage).getProperties().getProperty( CommonConstants.CONN_HOME_DIR_PROP );
		
			// Initialize selected execution with property
			if( selExecution == null )
				selExecution = EMPTY_STRING;
			labSelExecution.setText(selExecution);
		}
	}	
	
	/**
	 * Get selected execution 
	 * @return selected execution or an empty string if there is not selected execution
	 */
	String getExecutionSelection(){
		if(labSelExecution.getText().trim().compareTo("") == 0)
		{
			return EMPTY_STRING;
		}
		return labSelExecution.getText();
	}	
	
	
    Properties collectCustomProperties( Properties props )
    {
        if( props == null )
            props = new Properties();
        
        // set custom driver specific properties
		props.setProperty( CommonConstants.CONN_HOME_DIR_PROP,
				getExecutionSelection( ) );
		return props;
    }
    
    void initCustomControl( Properties profileProps )
    {
        if( profileProps == null || profileProps.isEmpty() || 
        		labSelExecution == null )
            return;     // nothing to initialize
        
        String executionSel = profileProps.getProperty( CommonConstants.CONN_HOME_DIR_PROP );
	    if( executionSel == null )
	    	executionSel = EMPTY_STRING;
	    labSelExecution.setText( executionSel );
    }
    
	/**
	 * Set message
	 * @param message
	 */
	private void setMessage( String message )
	{
		if ( m_wizardPage != null )
			m_wizardPage.setMessage( message );		
		else if ( m_propertyPage != null )
			m_propertyPage.setMessage( message );
	}
	
	/**
     * Checks if is page complete.
     * 
     * @return true, if is page complete
     */
	public boolean isPageComplete() {		
		return(completePage);		
	}
	
}
