/*
* CLIF is a Load Injection Framework
* Copyright (C) 2007 France Telecom
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF Jordan BRUNIER Grégory CALONNIER
*
* Contact: clif@ow2.org
*/

package org.ow2.clif.analyze.lib.oda.util.querytextutil;

import org.eclipse.datatools.connectivity.oda.OdaException;
import org.ow2.clif.analyze.lib.oda.i18n.Messages;

// TODO: Auto-generated Javadoc
/**
 * Utility class for splitting query text into query and columns information.
 */

public class QueryTextUtil
{

	/** The Constant QUERY_TEXT_DELIMITER. */
	private static final char QUERY_TEXT_DELIMITER = ':';

	/** The Constant COLUMNSINFO_BEGIN_DELIMITER. */
	private static final char COLUMNSINFO_BEGIN_DELIMITER = '{';

	/**
     * Instantiates a new query text util.
     */
	private QueryTextUtil( )
	{

	}

	
	///////////////////////////
	// public static methods //
	///////////////////////////
		
	/**
     * Gets the query.
     * 
     * @param queryText the query text
     * 
     * @return the query without the columns informations from the given query
     *         text
     * 
     * @throws OdaException the oda exception
     */
	public static String getQuery( String queryText ) throws OdaException
	{
		return splitQueryText( queryText )[0];
	}

	/**
     * Gets the columns info.
     * 
     * @param queryText the query text
     * 
     * @return the columns informations extracted from the given query text
     * 
     * @throws OdaException the oda exception
     */
	public static String getColumnsInfo( String queryText ) throws OdaException
	{
		assert queryText != null;
		return splitQueryText( queryText )[1];

	}

	
	/////////////////////
	// private methods //
	/////////////////////
		
	/**
     * Split query text.
     * 
     * @param queryText the query text
     * 
     * @return the string[]
     * 
     * @throws OdaException the oda exception
     */
	private static String[] splitQueryText( String queryText )
			throws OdaException
	{
		int delimiterIndex = -1;
		int columnsInfoBeginIndex = -1;
		
		String trimmedQueryText = queryText.trim( );
		
		String[] splittedQueryText = {
				"", "" //$NON-NLS-1$ //$NON-NLS-2$
		};
		boolean inQuote = false;
		boolean isEscaped = false;
		char[] chars = trimmedQueryText.toCharArray( );
		
		for ( int i = 0; i < chars.length; i++ )
		{
			if ( chars[i] == '"' )
			{
				if ( !isEscaped )
					inQuote = !inQuote;
				else
					isEscaped = !isEscaped;
			}
			else if ( chars[i] == '\\' )
			{
				isEscaped = !isEscaped;
			}
			else if ( ( !inQuote ) && chars[i] == QUERY_TEXT_DELIMITER )
			{
				delimiterIndex = i;
			}
			else if ( ( !inQuote ) && chars[i] == COLUMNSINFO_BEGIN_DELIMITER )
			{
				columnsInfoBeginIndex = i;
				break;
			}
		}

		if ( inQuote )
		{
			throw new OdaException( Messages.getString( "query_text_error" ) ); //$NON-NLS-1$
		}

		if ( delimiterIndex != -1 && columnsInfoBeginIndex != -1 )
		{
			splittedQueryText[0] = trimmedQueryText.substring( 0, delimiterIndex ).trim( );
			splittedQueryText[1] = trimmedQueryText.substring( columnsInfoBeginIndex + 1,trimmedQueryText.length( )-1 ).trim( );
		}
		else if ( delimiterIndex == -1 && columnsInfoBeginIndex == -1 )
		{
			splittedQueryText[0] = trimmedQueryText;
		}
		else
		{
			throw new OdaException( Messages.getString( "query_text_error" ) ); //$NON-NLS-1$
		}

		return splittedQueryText;
	}
}
