/*
 * CLIF is a Load Injection Framework
 * Copyright (C) 2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * CLIF $Name: not supported by cvs2svn $
 *
 * Contact: clif@ow2.org
 */

package org.ow2.clif.console.lib.egui.editor;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Shell;
import org.ow2.clif.storage.api.CollectListener;
import org.ow2.clif.deploy.ClifAppFacade;

/**
 * Progress monitor while collecting results
 * @author Joan Chaumont
 *
 */
public class CollectProgressDialog implements CollectListener, IRunnableWithProgress{
    protected IProgressMonitor monitor;
    
    protected boolean completed = false;
    protected boolean canceled = false;
    
    protected long fullsize = -1;
    protected long bladesize = -1;
    protected long progress = -1;
    
    protected ClifAppFacade clifApp;
    private String[] bladesId;
    
    private Shell s;
    
    /**
     * Constructor
     * @param shell parent shell
     * @param clifApp clifApp used by this test
     * @param bladesId blades id to collect
     */
    public CollectProgressDialog(Shell shell, ClifAppFacade clifApp, String[] bladesId) {
        this.s = shell;
        this.clifApp = clifApp;
        this.bladesId = bladesId;
    }
    
    /**
     * Run the ProgressMonitorDialog
     */
    public void go()
    {
        try {
            new ProgressMonitorDialog(s).run(true, true, this);
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
    
    /* Runnable interface */
    
    public void run(IProgressMonitor monitor) throws InterruptedException
    {
        this.monitor = monitor;
        clifApp.collect(bladesId, this);
    }

    /* CollectListener interface */
    
    public void collectStart(String testId, long size)
    {
        monitor.beginTask("Collecting test results for test " + testId, (int)size);
        fullsize = size;
        progress = 0;
        bladesize = 0;
        monitor.subTask("starting collecting " + size + " bytes for test " + testId);
    }
    
    public void bladeCollectStart(String bladeId, long size)
    {
        bladesize = size;
        monitor.subTask("starting collecting " + size + " bytes from blade " + bladeId);
    }
    
    public void progress(String bladeId, long done)
    {
        if (monitor.isCanceled()) {
            canceled = true;
        }
        monitor.subTask(
                "progress: " + done + "/" + bladesize + " for blade " + bladeId
                + ", " + (progress + done) + "/" + fullsize + " for full collect");
        monitor.worked((int)(done-progress));
        progress = done;
        
    }
    
    public void done()
    {
        monitor.subTask("done " + progress + "/" + fullsize);
        monitor.worked((int)progress);
        completed = true;
    }
    
    public boolean isCanceled()
    {
        return canceled;
    }
    
    public boolean isCanceled(String bladeId)
    {
        return canceled;
    }
}
