/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004, 2005 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.scenario.isac.egui.plugins.nodes;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.ow2.clif.scenario.isac.egui.plugins.gui.ParametersWidgetsNode;
import org.ow2.clif.scenario.isac.egui.plugins.gui.WidgetDescription;

/**
 * This class is an implementation of some methods wich will be used to build some
 * nodes of type 'choice'
 *
 * @author JC Meillaud
 * @author A Peyrard
 * @author Bruno Dillenseger
 */
public class ChoiceNode {

    /**
     * Create a node description for a choice
     * @return The node description for this controler
     */
    public static NodeDescription createNodeDescription() {
        /* create a new desc */
        NodeDescription desc = new NodeDescription(Node.CHOICE);
        Map<String,String> values = new HashMap<String,String>();
        values.put("proba","");
        desc.setParams(values);
        return desc ;
    }

    /**
     * Create the parameters widgets tree
     * @return The tree created
     */
    public static ParametersWidgetsNode createParametersWidgetsNode() {
        ParametersWidgetsNode pwn = new ParametersWidgetsNode(null) ;
        /* add the proba parameter field */
        ParametersWidgetsNode child = new ParametersWidgetsNode(new WidgetDescription(WidgetDescription.TEXT_FIELD, "proba",null, null)) ;
        pwn.addChild(child) ;
        return pwn ;
    }

    /**
     * This method return the help of an choice node
     * @return The help lines
     */
    public static Vector<String> getHelp() {
        Vector<String> help = new Vector<String>();
        help.add("This is a choice node:");
        help.add("This node defines a possible behavior branch in an nchoice statement.");
        help.add("It has a weight that rules the probability that this branch is executed rather than the other branches defined in the same nchoice statement.");
        help.add("The weight is a non-null positive integer value. The greater it is, the higher its execution probability.");
        return help ;
    }
}
