/*
* CLIF is a Load Injection Framework
* Copyright (C) 2004 France Telecom R&D
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
* CLIF 
*
* Contact: clif@ow2.org
*/
package org.ow2.clif.scenario.isac.egui.plugins.nodes;

import java.util.Vector;

import org.ow2.clif.scenario.isac.egui.plugins.PluginManager;

/**
 * This class implements some method which will be used when we will create some
 * controlers nodes
 * 
 * @author JC Meillaud
 * @author A Peyrard
 */
public class ControlerNode {

    /**
     * Get the help for the type of controler node given in parameter
     * @param type The type of the controler node
     * @return The help lines
     */
    public static Vector<String> getHelp(String type) {
        if (type.equals(Node.IF)) 
            return  IfNode.getHelp() ;
        if (type.equals(Node.WHILE))
            return WhileNode.getHelp() ;
        if (type.equals(Node.PREEMPTIVE)) 
            return PreemptiveNode.getHelp() ;
        return null ;
    }
    
    /**
     * This method create a node description for the controler node described in parameter
     * @param pluginManager the plugin manager
     * @param plugin The plugin name of the test
     * @param type The type of the node, which will be used a test
     * @param name The name of the test used
     * @return The node description for this controler node
     */
    public static NodeDescription createNodeDescription(PluginManager pluginManager, String plugin, String type, String name) {
        if (type.equals(Node.IF)) 
            return  IfNode.createNodeDescription(pluginManager, plugin, name) ;
        if (type.equals(Node.WHILE))
            return WhileNode.createNodeDescription(pluginManager, plugin, name) ;
        if (type.equals(Node.PREEMPTIVE)) 
            return PreemptiveNode.createNodeDescription(pluginManager, plugin, name) ;
        return null ;
    }
}
